/*
 * VBPJSTUB.C - First stage setup program for VB apps. Written by Michiel de Bruijn,
 *              published in Visual Basic Programmers Journal
 *
 *				Four-hundred-something lines of styleless chaos. Nicely commented, though...
 */

#include <stdlib.h>
#include <string.h> 
#include <direct.h>
#include <stdio.h>
#include <errno.h>
#include <windows.h>
#include <lzexpand.h>
#include "vbpjstub.h"

//Global vars and forward references
LRESULT CALLBACK VBPJStubWndProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
void DisplayText(HWND hWnd);
extern long int __far __pascal GetFreeDiskSpace(int DriveNumber);

const int SCRATCH_SIZE = 256;   
const int TEXT_STYLE = DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_NOCLIP | DT_NOPREFIX;  
char szDisplay[] = "DISPLAY";	//Section names in INI file
char szTemporary[] = "TEMPORARY FILES";
char szMessage[] = "MESSAGES";
char szScratch[256], szScratch2[256];
LPSTR lpszIniFileName;               
int iWindowWidth, iWindowHeight;

//Action starts here
int PASCAL WinMain(HANDLE hInst, HANDLE hInstPrev, LPSTR lpszMyCmdLine, int nCmdShow) {

    char szModuleName[_MAX_PATH], szFatalCaption[128], szSourcePath[_MAX_PATH];
    char szDestFile[_MAX_PATH], szSourceFile[_MAX_PATH], szSourceFileAlt[_MAX_PATH];
    char szNewIniFileName[_MAX_PATH], szFileKey[8], szNewDir[] = "C:\\~VBPJ$$$.X";
    char szCmdLine[128], szStubClass[] = "VBPJStubClass"; //Name of our window class
    char szNoRemoveMsg[512];
    LPSTR lpszExeFileName, lpszSourcePath, lpszNewDir, lpszDestFile;
    LPSTR lpszSourceFile, lpszSourceFileAlt, lpszCmdLine;
    size_t lDestLen;
    int iScreenWidth, iScreenHeight, iWindowX, iWindowY, iDrive, iDriveType;
    int iExt, iResult, iFile, bDone;
    long wReturn, wMinSpace;
    HWND hWnd; WNDCLASS wc; MSG msg;
    HFILE hfSource, hfDest; //File handles used by (LZ)copy
    OFSTRUCT ofSource, ofDest; //Open file structs used by same
    HINSTANCE hInstNew;
    	
	//Load string resource to use as a caption for "fatal error"-type message boxes
    LoadString(hInst, IDS_FATAL_ERR, szFatalCaption, 128);
     	 
 	/* =================================================================================
 	 *  PART 1: Get the source directory (the directory we're running from) and the
	 *			name of the .INI file to get our options from	 			
 	 * ================================================================================= 
 	 */
 
    //Get the full path to our .EXE
    if (!GetModuleFileName(hInst, szModuleName, _MAX_PATH)) {
    	LoadString(hInst, IDS_GETMOD_ERR, szScratch, SCRATCH_SIZE);
    	MessageBox(0, szScratch, szFatalCaption, MB_ICONSTOP); 
    	return 0xff;
    }
        	
    //Find final backslash and extract the EXE file name
    lpszExeFileName = _fstrrchr(szModuleName, '\\');
    if (lpszExeFileName) {
    	lpszExeFileName++;
    	//Next, create a string with the name of our INI file
    	lpszIniFileName = _fstrdup(szModuleName);
    	_fstrncpy(lpszIniFileName + _fstrlen(szModuleName) - 3, "INI", 3);
    }
    else {
    	LoadString(hInst, IDS_PARSE_ERR, szScratch, 256);
    	MessageBox(0, szScratch, szFatalCaption, MB_ICONSTOP); 
    	return 0xff;
    }                                          
    
    //The source path is everything to the left of the EXE name we just found...
    lDestLen = _fstrlen(szModuleName) - _fstrlen(lpszExeFileName);
    lpszSourcePath = _fstrncpy(szSourcePath, szModuleName, lDestLen);
    
    
    /* =================================================================================
 	 *  PART 2: Display a nice window to let the user know we're among the living
	 * ================================================================================= 
 	 */    

	//Get and calc positioning info    
	iScreenWidth = GetSystemMetrics(SM_CXSCREEN);
	iScreenHeight = GetSystemMetrics(SM_CYSCREEN);
	
	iWindowWidth = GetPrivateProfileInt(szDisplay, "WindowWidth", 300, lpszIniFileName);
	iWindowHeight = GetPrivateProfileInt(szDisplay, "WindowHeight", 150, lpszIniFileName);
	iWindowX = (iScreenWidth - iWindowWidth) / 2;
	iWindowY = (iScreenHeight - iWindowHeight) / 2;
	
	//Set up a nice window class for our app
	wc.style         = 0;
	wc.lpfnWndProc   = VBPJStubWndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInst;
	wc.hIcon         = LoadIcon(hInst, "Setup_Icon");
	wc.hCursor       = LoadCursor(hInst, IDC_WAIT);
	wc.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
	wc.lpszMenuName  = NULL;
    wc.lpszClassName = szStubClass;
    
    //Actually register the class
	if (!RegisterClass(&wc)) {
		LoadString(hInst, IDS_REGCLASS_ERR, szScratch, 256);
    	MessageBox(0, szScratch, szFatalCaption, MB_ICONSTOP); 
    	return 0xff;
    }

	//Get window caption from INI and create the window
	GetPrivateProfileString(szDisplay, "WindowCaption", "Setup", szScratch, SCRATCH_SIZE, lpszIniFileName);
	
	hWnd = CreateWindow (szStubClass, szScratch, WS_VISIBLE, iWindowX, iWindowY, iWindowWidth,
							 iWindowHeight, (HWND)NULL, (HMENU)NULL, hInst, NULL);
	
    if (!hWnd) {
    	LoadString(hInst, IDS_WIN_ERR, szScratch, 256);
    	MessageBox(0, szScratch, szFatalCaption, MB_ICONSTOP); 
    	return 0xff;
    }   

	//Draw window and get the device context    
	ShowWindow(hWnd, SW_SHOW);
	
	//Use DisplayText function to...
	DisplayText(hWnd);
	UpdateWindow(hWnd);
	
    /* =================================================================================
 	 *  PART 3: Find a suitable drive and create a temporary install dir there
	 * ================================================================================= 
 	 */    
    
    //Get min. required disk space for temp directory from INI
	wMinSpace = GetPrivateProfileInt(szTemporary, "MinSpace",  1024, lpszIniFileName);
	    
    //First, try the same drive Windows is on
	GetWindowsDirectory(szScratch, SCRATCH_SIZE);
	iDrive = (int)szScratch[0] - 64;
	
	if (GetFreeDiskSpace(iDrive) < wMinSpace) {            
		iDrive = 1;
RetryDrives:
		//Not enough space on Windows drive. Now try the others...
		for(wReturn = 0; (iDrive <= 26) && (wReturn != 1); iDrive++) {
			iDriveType = GetDriveType(iDrive);
			if ((iDriveType != DRIVE_REMOVABLE) && (iDriveType != 0))
				if (GetFreeDiskSpace(iDrive) >= wMinSpace)
					break;			
		}
	}
	
	//Bail out if we still haven't got something usable by now...
	if(GetFreeDiskSpace(iDrive) < wMinSpace) {
		GetPrivateProfileString(szMessage, "NoTempSpace", "!No temp space", szScratch, SCRATCH_SIZE, lpszIniFileName);
		MessageBox(0, szScratch, szFatalCaption, MB_ICONSTOP);
		goto CleanUp;
	}
			
	//Try to create a temporary directory on the drive we just found
	GetPrivateProfileString(szTemporary, "TempRoot", "~VBPJ$$$", szScratch, SCRATCH_SIZE, lpszIniFileName);
    //Try max. 26 times
    for(iExt = 0, iResult = 1; (iExt <= 26) && (iResult != 0) ; iExt++) { 
		szNewDir[0] = iDrive + 64; szNewDir[12] = iExt + 65;
		lpszNewDir = _fstrncpy(szNewDir + 3, szScratch, 8);
		iResult = _mkdir(szNewDir);
	}
	
	//If the directory couldn't be created, this is probably a network drive where we don't have write
	//permission in the root or some other kind of write-protected drive. Resume the scan...
	if(iResult != 0) {
		iDrive++;
		goto RetryDrives; 		
	}
    
    /* =================================================================================
 	 *  PART 4: Copy all files specified in our INI file to the temporary directory
	 * ================================================================================= 
 	 */    
	
	//Init source/destination files	
	_fstrncpy(szSourceFile, szSourcePath, strlen(szSourcePath));
	_fstrncpy(szDestFile, szNewDir, strlen(szNewDir));
	
	//Init data compression stuff
	LZStart();
	
	//Copy max. 99 files
	for(iFile = 1, bDone = 0; (iFile <= 99) && (bDone == 0); iFile++) {
		sprintf(szFileKey, "FILE%d", iFile);
		GetPrivateProfileString(szTemporary, szFileKey, "*", szScratch, SCRATCH_SIZE, lpszIniFileName);
		if(szScratch[0] != '*') {
			
			//Add filename to destination path
			_fstrncpy(szDestFile + strlen(szNewDir), "\0", 1);
			_fstrcat(szDestFile, "\\");	lpszDestFile = _fstrcat(szDestFile, szScratch);
			
			//Add filename to source path
			_fstrncpy(szSourceFile + strlen(szSourcePath), "\0", 1);
			lpszSourceFile = _fstrcat(szSourceFile, szScratch);     
			//Alternate source file: compressed filename convention (file.ex_)
			_fstrcpy(szSourceFileAlt, szSourceFile);                
			_fstrncpy(szSourceFileAlt + strlen(szSourcePath), "\0", 1);
			szScratch[strlen(szScratch) - 1] = '_';
			lpszSourceFileAlt = _fstrcat(szSourceFileAlt, szScratch);			
			
			//Open source file for reading
			hfSource = LZOpenFile(lpszSourceFile, &ofSource, OF_READ);
			if(hfSource == -1) {
				hfSource = LZOpenFile(lpszSourceFileAlt, &ofSource, OF_READ);
				if(hfSource == -1) {
					GetPrivateProfileString(szMessage, "OpenFail", "!Open Failure on %s", szScratch, SCRATCH_SIZE, lpszIniFileName);
					//Insert filename into error message
					sprintf(szScratch2, szScratch, szSourceFile);
					MessageBox(0, szScratch2, szFatalCaption, MB_ICONEXCLAMATION); DisplayText(hWnd);
				}
			}
						
			if(hfSource != -1) {
			
				hfDest = LZOpenFile(lpszDestFile, &ofDest, OF_CREATE);
			
				if(CopyLZFile(hfSource, hfDest) < 0) {
					GetPrivateProfileString(szMessage, "CopyFail", "!Copy Failure on %s", szScratch, SCRATCH_SIZE, lpszIniFileName);
					//Insert filename into error message
					sprintf(szScratch2, szScratch, szSourceFile);
					MessageBox(0, szScratch2, szFatalCaption, MB_ICONEXCLAMATION); DisplayText(hWnd);
					//Delete (empty/corrupted) destination file
			    	LZClose(hfDest); unlink(szDestFile);
			    }   
			}
			
			LZClose(hfSource); LZClose(hfDest);
			
		}
		else
			bDone = 1;
	}
	
	LZDone();		
	
	/* =================================================================================
 	 *  PART 5: Launch the second-stage setup program as specified in our INI file
 	 *			We pass the source directory on the command line: the app can determine
 	 *			the temporary directory by using the App.Path property			
	 * ================================================================================= 
 	 */    
    
    //Start with the temporary directory path
    _fstrncpy(szCmdLine, szNewDir, strlen(szNewDir));
    
    //Get name of setup executable from INI
    GetPrivateProfileString(szTemporary, "SetupEXE", "SETUP1.EXE", szScratch, SCRATCH_SIZE, lpszIniFileName);
    
    //And add that to the command line
	_fstrncpy(szCmdLine + strlen(szNewDir), "\0", 1);
	_fstrcat(szCmdLine, "\\"); _fstrcat(szCmdLine, szScratch);
	//Now, add a space and the name of the source directory
	_fstrcat(szCmdLine, " "); lpszCmdLine = _fstrcat(szCmdLine, szSourcePath);
    
	//Go and run this thing...	    
	hInstNew = WinExec(lpszCmdLine, SW_SHOW);        

    //Hide our window (no need for it anymore)
	ShowWindow(hWnd, SW_HIDE);
	
	//Bother the user if the SetupEXE failed to start properly
	if(hInstNew < 32) {
		GetPrivateProfileString(szMessage, "LaunchFail", "!Couldn't Start %s", szScratch, SCRATCH_SIZE, lpszIniFileName);
		//Insert filename into error message
		sprintf(szScratch2, szScratch, szCmdLine);
		MessageBox(0, szScratch2, szFatalCaption, MB_ICONSTOP);
		goto CleanUp;
	}
								
	/* =================================================================================
 	 *  PART 6: If specified in the INI file, wait until the second-stage setup app
 	 *			terminates and clean up the temporary directory after it.
 	 * ================================================================================= 
 	 */    
    
     if(GetPrivateProfileInt(szTemporary, "CleanUp",  1, lpszIniFileName) != 0) {
     	
     	//Set up timer to send our window messages from time to time
     	if(!SetTimer(hWnd, 1, 1500, VBPJStubWndProc)) {
     		//In the (rare) event no timer is available, bail out (would starve otherwise)
     		// "couldn't remove directory" message is immediately displayed
     		goto NoTimerBailOut;
     	}
     	
     	//Wait for SetupEXE to terminate
     	while(GetModuleUsage(hInstNew)) {
			//GetMessage loop to keep system running
			GetMessage(&msg, hWnd, 0, 0);
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		
		//Change path to INI file to point to temporary directory
		// (disk 1 is no longer guaranteed to be in the drive, remember?)
		_fstrcpy(szNewIniFileName, szNewDir);
		_fstrcat(szNewIniFileName, "\\");
		lpszIniFileName = _fstrcat(szNewIniFileName, lpszExeFileName);
		_fstrncpy(lpszIniFileName + _fstrlen(lpszIniFileName) - 3, "INI", 3);
				
		//Get "couldn't remove directory" message from INI before it's too late
		GetPrivateProfileString(szMessage, "NoRemove", "!Error Removing Directory '%s'", szNoRemoveMsg, 512, lpszIniFileName);
		
		//Get files again from the INI file				
		for(iFile = 1, bDone = 0; (iFile <= 99) && (bDone == 0); iFile++) {
			sprintf(szFileKey, "FILE%d", iFile);
			GetPrivateProfileString(szTemporary, szFileKey, "*", szScratch, SCRATCH_SIZE, lpszIniFileName);
			if(szScratch[0] != '*') {
			
				//Add filename to destination path
				_fstrncpy(szDestFile + strlen(szNewDir), "\0", 1);
				_fstrcat(szDestFile, "\\");	lpszDestFile = _fstrcat(szDestFile, szScratch);
				
				//Remove file
				unlink(szDestFile);				
				
			}
			else
				bDone = 1;
		
		}   
		
NoTimerBailOut:		
		//Try to remove the temporary directory and bug user if impossible
		_rmdir(szNewDir);		
		if(errno == EACCES) {
			sprintf(szScratch, szNoRemoveMsg, szNewDir);
			MessageBox(0, szScratch, "Setup", MB_ICONINFORMATION);
		}
								
     }
     
CleanUp:	
	//Finally, destroy our window and un-register the window class
	DestroyWindow(hWnd);
	UnregisterClass(szStubClass, hInst);

	return 0x00;
	
}


/* =================================================================================
 *  APPENDICES: Text drawing & window procedure 'helper' functions
 * ================================================================================= 
 */    


//Draw text from INI onto our window
void DisplayText(HWND hWnd) {
	
	HDC hDC; //Device context to draw window text onto
    RECT rc; //Rectangle used to draw text
    	
	hDC = GetDC(hWnd);
	SetBkMode(hDC, TRANSPARENT);
	
	//Output first line of text
	GetPrivateProfileString(szDisplay, "WindowLine1", "Starting Setup", szScratch, SCRATCH_SIZE, lpszIniFileName);
	rc.left = 0; rc.top = 15; rc.right = iWindowWidth; rc.bottom = iWindowHeight /2;
	DrawText(hDC, szScratch, strlen(szScratch), &rc, TEXT_STYLE);
	                           
	//Output second line of text
	GetPrivateProfileString(szDisplay, "WindowLine2", "Please Wait...", szScratch, SCRATCH_SIZE, lpszIniFileName);
	rc.left = 0; rc.top = rc.bottom; rc.right = iWindowWidth; rc.bottom = iWindowHeight - 40;
	DrawText(hDC, szScratch, strlen(szScratch), &rc, TEXT_STYLE);

}	

//Shortest window procedure in the world...
LRESULT CALLBACK VBPJStubWndProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam) {
    
    switch(Msg) {
    	case WM_PAINT:
    		DisplayText(hWnd);
    		        
    	default:    
			return (DefWindowProc(hWnd, Msg, wParam, lParam));	
	}
	
}
