/****************************************************************************
*
*		Thread DLL for Visual Basic Programmer's Journal Article
*		Copyright  1996 Stephen R. Husak
*
*		PROJECT		:	Thread
*		FILENAME	:	thread.c
*		AUTHOR		:	12/26/95	SRH
*
*		PURPOSE		:	main file containing implementation functions of DLL
*								     
*		REVISIONS	:	12/26/95	SRH		wrote DLLMain Function and Thread Function
*								12/27/95	SRH		wrote DirBrowse Function
*								01/01/96	SRH		changed DirData structure for multiple parms
*								01/05/96	SRH		rewrote DirBrowse Function logic checks
*																commented everything fully
*        
****************************************************************************/

/*
 * Standard Include Files
 */
#include <windows.h>						// standard windows include
#include <windowsx.h>						// message crackers include
#include <stdio.h>							// standard i/o functions (sprintf)

/*
 * Application Include Files
 */
#include "thread.h"							// application include file

/***************************************************************************
 *	FUNCTION :	DLLMain
 *							(HINSTANCE hinstDLL,		instance handle to the DLL
 *							 DWORD fdwReason,				flag specifying how DLL was loaded
 *							 LPVOID lpvReserved)		flag specifying if DLL was loaded 
 *																			implicitly or explicitly
 *
 *  RETURN   :	BOOL				success or failure of DLLMain function
 *
 *  PURPOSE  :	this is the entry point to the DLL; it is called when the
 *							library is first loaded and during attaching and detaching
 *							of threads
 **************************************************************************/ 
BOOL WINAPI DllMain(HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved) {

	switch (fdwReason) {
	  case DLL_PROCESS_ATTACH:
		  // DLL is attaching to the address space of the current process
		break;
	    
	  case DLL_THREAD_ATTACH:
		  // a new thread is being created in the current process
	  break;

		case DLL_THREAD_DETACH:
		  // a thread is exiting cleanly
	 	break;

  	case DLL_PROCESS_DETACH:
      // the calling process is detaching the DLL from its address space
    break;
	}
	return (TRUE);			// this DLL has no reason to fail in loading
}

/***************************************************************************
 *	FUNCTION :	DirBrowseThread
 *							(void * lpThreadParm)		parameters passed to the thread 
 *																			(DIRDATA structure)
 *
 *  RETURN   :	DWORD				exit code of thread function
 *
 *  PURPOSE  :	this is the start of the directory browse thread 
 **************************************************************************/ 
DLLEXPORT DWORD WINAPI DirBrowseThread(DIRDATA * dd) {

	char * sPath;				// buffer for path
	char * sFileSpec;		// buffer for file specification

	// allocate buffers and adjust character widths passed from VB
	sPath = (char *) malloc(dd->lPathLen + 1);
	strncpy(sPath, dd->path, dd->lPathLen);
	sPath[dd->lPathLen] = 0;

	sFileSpec = (char *) malloc(dd->lFileSpecLen + 1);
	strncpy(sFileSpec, dd->filespec, dd->lFileSpecLen);
	sFileSpec[dd->lFileSpecLen] = 0;

	// set global variables
	gl_DirCount = 0;
	gl_FileCount = 0;

	//MessageBox(NULL, "Thread has Started!", "DLL Thread", MB_OK);

	// start the browse
	DirBrowse(sPath, sFileSpec, dd->hWndList, dd->hWndDirs, dd->hWndFiles);

	// deallocate buffers & reset global variables
	free(sPath); free(sFileSpec);
	gl_DirCount = 0;
	gl_FileCount = 0;

	//MessageBox(NULL, "Thread About to End!", "DLL Thread", MB_OK);

	// exit the thread
	ExitThread(ERROR_SUCCESS);

	return (ERROR_SUCCESS);
}

/***************************************************************************
 *	FUNCTION :	DirBrowse
 *							(LPSTR path,				path to search on
 *							 LPSTR filespec,		file specification (which files to look for)
 *							 HWND hWndList,			handle to list box to stuff names
 *							 HWND hWndDirs,			handle to text box for folder count
 *							 HWND hWndFiles)		handle to text box for file count
 *
 *  RETURN   :	void
 *
 *  PURPOSE  :	this is the directory browse function that does the 
 *							recursive calls to retrieve the directory of the specified
 *							drive
 **************************************************************************/ 
DLLEXPORT void WINAPI DirBrowse(LPSTR path, LPSTR filespec, HWND hWndList,
																HWND hWndDirs, HWND hWndFiles) {
		
	WIN32_FIND_DATA finddata;			// finddata structure filled by calls
	HANDLE fSearch;								// handle to the file find instance
	BOOL rc = TRUE;								// boolean return code fails when FindNextFile fails
	char work[MAX_PATH];					// work buffer to build paths and file specs
	
	// build the string required by FindFirstFile (path\filespec eg. c:\*.*)
	sprintf(work, "%s\\%s", path, filespec);

	// find the first file matching the spec
	fSearch = FindFirstFile(work, &finddata);

	while (rc && (fSearch != INVALID_HANDLE_VALUE)) {
		
		// weed out directories return as . and ..
		if (finddata.cFileName[0] != '.') {
			// send the name to the listbox 
			sprintf(work, "%s\\%s", path, finddata.cFileName);
			ListBox_AddString(hWndList, work);
		
			if (finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
				// this is a directory so count it and recurse its structure
				sprintf(work, "%s\\%s", path, finddata.cFileName);
				gl_DirCount += 1;

				DirBrowse(work, filespec, hWndList, hWndDirs, hWndFiles);
			}
			else	// it is a file
				gl_FileCount += 1;
		}

		// report stats to textboxes
		sprintf(work, "%i", gl_DirCount);
		Edit_SetText(hWndDirs, work);
		
		sprintf(work, "%i", gl_FileCount);
		Edit_SetText(hWndFiles, work);
		
		// get next matching file
		rc = FindNextFile(fSearch, &finddata);
	}

	// we've gotten all files so close search and exit function
	FindClose(fSearch);
	return;
}

/* EOF: thread.c */