Attribute VB_Name = "EventLog"
Option Explicit


' -------------------------------------------------
'
' -------------------------------------------------
Public Const EVNT_SYSTEM = "System"
Public Const EVNT_APP = "Application"
Public Const EVNT_SECURITY = "Security"


' -------------------------------------------------
'
' -------------------------------------------------
Private Const FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200
Private Const FORMAT_MESSAGE_FROM_STRING = &H400
Private Const FORMAT_MESSAGE_FROM_HMODULE = &H800
Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const FORMAT_MESSAGE_ARGUMENT_ARRAY = &H2000
Private Const FORMAT_MESSAGE_MAX_WIDTH_MASK = &HFF


' -------------------------------------------------
'
' -------------------------------------------------
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006


' -------------------------------------------------
'
' -------------------------------------------------
Type EVENTLOGRECORD
   Length               As Long     ' Length of full record
   Reserved             As Long     ' Used by the service
   RecordNumber         As Long     ' Absolute record number
   TimeGenerated        As Long     ' Seconds since 1-1-1970
   TimeWritten          As Long     ' Seconds since 1-1-1970
   EventID              As Long
   EventType            As Integer
   NumStrings           As Integer
   EventCategory        As Integer
   ReservedFlags        As Integer  ' For use with paired events (auditing)
   ClosingRecordNumber  As Long     ' For use with paired events (auditing)
   StringOffset         As Long     ' Offset from beginning of record
   UserSidLength        As Long
   UserSidOffset        As Long
   DataLength           As Long
   DataOffset           As Long     '  Offset from beginning of record
   
   SourceName           As Variant
   ComputerName         As Variant
   SID                  As Variant
   TChar                As Variant
   Bytes                As Variant
   Pad                  As Variant
   EndLength            As Long
End Type


' -------------------------------------------------
'
' -------------------------------------------------
Type SYSTEMTIME
   wYear          As Integer
   wMonth         As Integer
   wDayOfWeek     As Integer
   wDay           As Integer
   wHour          As Integer
   wMinute        As Integer
   wSecond        As Integer
   wMilliseconds  As Integer
End Type

Type TIME_ZONE_INFORMATION
   Bias              As Long
   StandardName      As String * 32
   StandardDate      As SYSTEMTIME
   StandardBias      As Long
   DaylightName      As String * 32
   DaylightDate      As SYSTEMTIME
   DaylightBias      As Long
End Type


' -------------------------------------------------
' 32-bit API declares
' -------------------------------------------------
Private Declare Function OpenEventLog Lib "advapi32" Alias "OpenEventLogA" (ByVal lpUNCServerName As String, ByVal lpSourceName As String) As Long
Private Declare Function GetNumberOfEventLogRecords Lib "advapi32.dll" (ByVal hEventLog As Long, NumberOfRecords As Long) As Long
Private Declare Function ReadEventLog Lib "advapi32.dll" Alias "ReadEventLogA" (ByVal hEventLog As Long, ByVal dwReadFlags As Long, ByVal dwRecordOffset As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, pnBytesRead As Long, pnMinNumberOfBytesNeeded As Long) As Long
Private Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (dst As Any, src As Any, ByVal Size As Long)
Private Declare Function LoadLibrary Lib "kernel32" Alias "LoadLibraryA" (ByVal lpLibFileName As String) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" _
   (ByVal dwFlags As Long, lpSource As Any, _
    ByVal dwMessageId As Long, ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, ByVal nSize As Long, _
    Arguments As Long) As Long
Private Declare Function GetWindowsDirectory Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Private Declare Function GetTimeZoneInformation Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32" Alias _
   "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, _
   ByVal ulOptions As Long, ByVal samDesired As Long, _
   phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32" Alias _
   "RegQueryValueExA" (ByVal hKey As Long, _
   ByVal lpValueName As String, lpReserved As Long, _
   lptype As Long, lpData As Any, lpcbData As Long) As Long
   

' -------------------------------------------------
' Defines for the READ flags for Eventlogging
' -------------------------------------------------
Private Const EVENTLOG_SEQUENTIAL_READ = &H1
Private Const EVENTLOG_SEEK_READ = &H2
Private Const EVENTLOG_FORWARDS_READ = &H4
Private Const EVENTLOG_BACKWARDS_READ = &H8


' -------------------------------------------------
' The types of events that can be logged.
' -------------------------------------------------
Private Const EVENTLOG_SUCCESS = &H0
Private Const EVENTLOG_ERROR_TYPE = &H1
Private Const EVENTLOG_WARNING_TYPE = &H2
Private Const EVENTLOG_INFORMATION_TYPE = &H4
Private Const EVENTLOG_AUDIT_SUCCESS = &H8
Private Const EVENTLOG_AUDIT_FAILURE = &H10


' -------------------------------------------------
' Defines for the WRITE flags used by Auditing for
'   paired events
' These are not implemented in Product 1
' -------------------------------------------------
Private Const EVENTLOG_START_PAIRED_EVENT = &H1
Private Const EVENTLOG_END_PAIRED_EVENT = &H2
Private Const EVENTLOG_END_ALL_PAIRED_EVENTS = &H4
Private Const EVENTLOG_PAIRED_EVENT_ACTIVE = &H8
Private Const EVENTLOG_PAIRED_EVENT_INACTIVE = &H10


' --------------------------------------------------------
' Eventlog Status Codes
' --------------------------------------------------------
Private Const ERROR_SUCCESS = 0&
Private Const ERROR_FILE_NOT_FOUND = 2&
Private Const ERROR_PATH_NOT_FOUND = 3&
Private Const ERROR_TOO_MANY_OPEN_FILES = 4&
Private Const ERROR_ACCESS_DENIED = 5&
Private Const ERROR_INVALID_HANDLE = 6&
Private Const ERROR_ARENA_TRASHED = 7&
Private Const ERROR_NOT_ENOUGH_MEMORY = 8&
Private Const ERROR_OUTOFMEMORY = 14&
Private Const ERROR_HANDLE_EOF = 38&             'Reached end of file
Private Const ERROR_INVALID_PARAMETER = 87&
Private Const ERROR_INSUFFICIENT_BUFFER = 122&
Private Const ERROR_INVALID_NAME = 123&
Private Const ERROR_ALREADY_EXISTS = 183&
Private Const ERROR_MORE_DATA = 234&
Private Const ERROR_NO_MORE_ITEMS = 259&
Private Const ERROR_NOACCESS = 998&
Private Const ERROR_BADDB = 1009&
Private Const ERROR_BADKEY = 1010&
Private Const ERROR_CANTOPEN = 1011&
Private Const ERROR_CANTREAD = 1012&
Private Const ERROR_CANTWRITE = 1013&
Private Const ERROR_EVENTLOG_FILE_CORRUPT = 1500&
Private Const ERROR_EVENTLOG_CANT_START = 1501&
Private Const ERROR_LOG_FILE_FULL = 1502&
Private Const ERROR_EVENTLOG_FILE_CHANGED = 1503&
Private Const ERROR_RESOURCE_DATA_NOT_FOUND = 1812&


' -------------------------------------------------
' Read/Write permissions for registry
' -------------------------------------------------
Private Const KEY_QUERY_VALUE = &H1&
Private Const KEY_SET_VALUE = &H2&
Private Const KEY_CREATE_SUB_KEY = &H4&
Private Const KEY_ENUMERATE_SUB_KEYS = &H8&
Private Const KEY_NOTIFY = &H10&
Private Const KEY_CREATE_LINK = &H20&
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_READ = READ_CONTROL
Private Const STANDARD_RIGHTS_WRITE = READ_CONTROL
Private Const STANDARD_RIGHTS_EXECUTE = READ_CONTROL
Private Const KEY_READ = STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY
Private Const KEY_WRITE = STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY
Private Const KEY_EXECUTE = KEY_READ




Public Sub Main()
   ReadToFile EVNT_APP, "C:\EvntLog1.Txt"
   'ReadToFile EVNT_SYSTEM, "C:\EvntLog2.Txt"
   'ReadToFile EVNT_SECURITY, "C:\EvntLog3.Txt"
End Sub

' -------------------------------------------------
'
' -------------------------------------------------
Public Sub ReadToFile(xstrEventLogType As String, xstrFileNamePath As String)
   Dim pstrSid                As String
   Dim plngModHwnd            As Long
   Dim pstrSysDir             As String
   Dim pbyteBuffer()          As Byte
   Dim pstrMsgArg()           As String
   Dim pstrBuffer             As String
   Dim plngEventLogHwd        As Long
   Dim plngReadFlags          As Long
   Dim plngReadRecordOffset   As Long
   Dim plngNumBytesToRead     As Long
   Dim plngBytesRead          As Long
   Dim plngRecordNumber       As Long
   Dim plngMinNumBytesNeeded  As Long
   Dim pvntStartDate          As Variant
   Dim pvntAddDate            As Variant
   Dim plngBiasDate           As Long
   Dim plngEventType          As Long
   Dim plngRtn                As Long
   Dim pintLoop               As Integer
   Dim k                      As Integer
   Dim plngPos                As Long
   Dim pstrLibName            As String
   Dim plngFlags              As Long
   Dim plngSize               As Long
   Dim pstrMsgString          As String
   Dim plngFileNum            As Long
   
   
   ' Used to get info from returned string
   Dim plngRecLen             As Long
   Dim plngRecNum             As Long
   Dim pvntTimeGenerated      As Variant
   Dim pvntTimeWritten        As Variant
   Dim plngEventID            As Long
   Dim plngEventIdForFile     As Long
   Dim pintEventType          As Integer
   Dim pintNumStrings         As Integer
   Dim pintEventCategory      As Integer
   Dim plngClosingRecNumber   As Long
   Dim plngStringOffset       As Long
   Dim plngSidLength          As Long
   Dim plngSidOffset          As Long
   Dim plngDataLength         As Long
   Dim plngDataOffset         As Long
   Dim pstrEventType          As String
   Dim pstrTmp                As String
   Dim pstrSourceName         As String
   Dim pstrComputerName       As String
   Dim pstrStrings            As String
   Dim pstrData               As String
   
   
   ' ----------------------------------------------
   ' Open the log file we are going to write to
   ' ----------------------------------------------
   plngFileNum = FreeFile
   Open xstrFileNamePath For Output As #plngFileNum
   

   ' ----------------------------------------------
   ' Set the starting date and bias (ie, Time Zone)
   '   to figure the dates
   ' ----------------------------------------------
   pvntStartDate = CDate("01/01/1970 00:00:00")
   plngBiasDate = GetTimeBias()
   
   
   ' ----------------------------------------------
   ' Open the event log
   ' ----------------------------------------------
   plngEventLogHwd = OpenEventLog("", xstrEventLogType)

   
   ' ----------------------------------------------
   ' Get the number of records in this event log
   ' ----------------------------------------------
   If GetNumberOfEventLogRecords(plngEventLogHwd, _
                  plngRecordNumber) <> False Then
      Print #plngFileNum, CStr(plngRecordNumber) & " records in "; xstrEventLogType
      Print #plngFileNum, ""
   Else
      Print #plngFileNum, "Error getting number of Event Log Records: " & Err.LastDllError
      Exit Sub
   End If
   
   If plngRecordNumber <= 0 Then
      Print #plngFileNum, "No records in event log"
      Exit Sub
   End If
   
   
   ' ----------------------------------------------
   ' Set the read flags and the initial buffer
   ' ----------------------------------------------
   plngReadFlags = EVENTLOG_SEEK_READ Or EVENTLOG_FORWARDS_READ
   
   
   ' ----------------------------------------------
   ' Loop thru the record numbers
   ' ----------------------------------------------
   For pintLoop% = 1 To plngRecordNumber
      plngReadRecordOffset = pintLoop%
      
      plngNumBytesToRead = 56
      ReDim pbyteBuffer(0 To plngNumBytesToRead - 1)

TryAgain:
      plngRtn = ReadEventLog(plngEventLogHwd, _
         plngReadFlags, plngReadRecordOffset, _
         pbyteBuffer(0), plngNumBytesToRead, _
         plngBytesRead, plngMinNumBytesNeeded)
      
      If plngRtn = False Then
         If Err.LastDllError = ERROR_INSUFFICIENT_BUFFER Then
            plngNumBytesToRead = plngMinNumBytesNeeded
            ReDim pbyteBuffer(0 To plngNumBytesToRead - 1)
            GoTo TryAgain
         ElseIf Err.LastDllError = ERROR_HANDLE_EOF Then
            Exit For
         Else
            Print #plngFileNum, "Error reading the Event log: " & Err.LastDllError
            Exit For
         End If
      End If
      
      
      ' ----------------------------------------
      ' Copy the memory to a string
      ' ----------------------------------------
      pstrBuffer = Space$(plngBytesRead)
      CopyMem ByVal pstrBuffer, pbyteBuffer(0), plngBytesRead
      
      
      ' ----------------------------------------
      ' Get the info from the event log
      ' ----------------------------------------
      plngRecLen = CVL(Mid$(pstrBuffer, 1, 4))
      plngRecNum = CVL(Mid$(pstrBuffer, 9, 4))
      pvntAddDate = CVL(Mid$(pstrBuffer, 13, 4))
      pvntTimeGenerated = DateAdd("s", CLng(pvntAddDate) - plngBiasDate, pvntStartDate)
      pvntAddDate = CVL(Mid$(pstrBuffer, 17, 4))
      pvntTimeWritten = DateAdd("s", CLng(pvntAddDate) - plngBiasDate, pvntStartDate)
      plngEventID = CVL(Mid$(pstrBuffer, 21, 2))
      plngEventIdForFile = CVL(Mid$(pstrBuffer, 21, 4))
      'Debug.Print plngEventID
      pintEventType = CVI(Mid$(pstrBuffer, 25, 2))
      pintNumStrings = CVI(Mid$(pstrBuffer, 27, 2))
      pintEventCategory = CVI(Mid$(pstrBuffer, 29, 2))
      plngStringOffset = CVL(Mid$(pstrBuffer, 37, 4))
      plngSidLength = CVL(Mid$(pstrBuffer, 41, 4))
      plngSidOffset = CVL(Mid$(pstrBuffer, 45, 4))
      plngDataLength = CVL(Mid$(pstrBuffer, 49, 4))
      plngDataOffset = CVL(Mid$(pstrBuffer, 53, 4))
      
      
      ' ----------------------------------------
      ' Get the SourceName and ComputerName
      ' ----------------------------------------
      pstrTmp = Mid$(pstrBuffer, 57)
      On Error Resume Next
      plngPos = InStr(pstrTmp, Chr$(0))
      If plngPos > 0 Then
         pstrSourceName = Mid$(pstrTmp, 1, plngPos - 1)
      End If
      
      If plngPos > 0 Then
         pstrTmp = Mid$(pstrTmp, plngPos + 1)
         plngPos = InStr(pstrTmp, Chr$(0))
         If plngPos > 0 Then
            pstrComputerName = Mid$(pstrTmp, 1, plngPos - 1)
         End If
      End If
      
      
      ' ----------------------------------------
      ' Read the strings in the resource file
      ' ----------------------------------------
      If Len(Trim$(pstrSourceName)) > 0 Then
         pstrLibName = GetRegistryEntry(xstrEventLogType, pstrSourceName)
         
         pstrSysDir = Space$(512)
         plngRtn = GetWindowsDirectory(pstrSysDir, Len(pstrSysDir))
         pstrSysDir = Mid$(pstrSysDir, 1, plngRtn)
         
         
         pstrLibName = UCase$(pstrLibName)
         plngPos = InStr(pstrLibName, "%SYSTEMROOT%")
         If plngPos > 0 Then
            pstrLibName = pstrSysDir & Mid$(pstrLibName, plngPos + 12)
         End If
          
                     
         If pintNumStrings > 0 Then
            pstrStrings = Mid$(pstrBuffer, plngStringOffset + 1, (plngDataOffset - plngStringOffset))
            ReDim pstrMsgArg(1 To pintNumStrings)
            On Error Resume Next
            k% = 1
            Do
               plngPos = InStr(pstrStrings, Chr$(0))
               If plngPos > 0 Then
                  pstrMsgArg(k%) = Mid$(pstrStrings, 1, plngPos - 1)
                  pstrStrings = Mid$(pstrStrings, plngPos + 1)
                  k% = k% + 1
               End If
            Loop While plngPos > 0
            pstrStrings = ""
            For k% = 1 To pintNumStrings
               If k% = pintNumStrings Then
                  pstrStrings = pstrStrings & pstrMsgArg(k%)
               ElseIf k% = 1 Then
                  pstrStrings = pstrMsgArg(k%) & vbCr
               Else
                  pstrStrings = pstrStrings & pstrMsgArg(k%) & vbCr
               End If
            Next k%
            On Error GoTo 0
         End If
   
   
         ' -------------------------------------
         '
         ' -------------------------------------
         plngModHwnd = LoadLibrary(pstrLibName)

         If plngModHwnd > 0 Then
            
            ' ----------------------------------
            '
            ' ----------------------------------
            plngFlags = FORMAT_MESSAGE_FROM_HMODULE Or _
                        FORMAT_MESSAGE_IGNORE_INSERTS
            plngSize = 10240
            pstrMsgString = Space$(plngSize)


            ' ----------------------------------
            '
            ' ----------------------------------
            plngRtn = FormatMessage(plngFlags, ByVal plngModHwnd, _
                  CLng(plngEventIdForFile), &H0&, _
                  pstrMsgString, plngSize, _
                  ByVal 0)
            
            
            ' ----------------------------------
            '
            ' ----------------------------------
            If plngRtn <> 0 Then
               pstrMsgString = Mid$(pstrMsgString, 1, plngRtn)
               If pintNumStrings > 0 Then
                  ReplaceParameters pintNumStrings, pstrMsgString, pstrMsgArg()
               Else
                  pstrMsgString = ClearLastCrLf(pstrMsgString)
               End If
            Else
               pstrMsgString = ""
               If Err.LastDllError = ERROR_RESOURCE_DATA_NOT_FOUND Then
                  Print #plngFileNum, "Error with FormatMessage function: Resource Data not Found!"
               Else
                  Print #plngFileNum, "Error with FormatMessage function: " & Err.LastDllError
               End If
            End If
            
            
            ' ----------------------------------
            '
            ' ----------------------------------
            FreeLibrary plngModHwnd
         
         End If
         
         
         plngEventType = pintEventType
         Select Case plngEventType
            Case EVENTLOG_SUCCESS
               pstrEventType = "Success"
            Case EVENTLOG_ERROR_TYPE
               pstrEventType = "Error"
            Case EVENTLOG_WARNING_TYPE
               pstrEventType = "Warning"
            Case EVENTLOG_INFORMATION_TYPE
               pstrEventType = "Information Event"
            Case EVENTLOG_AUDIT_SUCCESS
               pstrEventType = "Audit Success"
            Case EVENTLOG_AUDIT_FAILURE
               pstrEventType = "Audit Failure"
         End Select
      End If
   
   
      ' -------------------------------------------
      ' Print the info to the file
      ' -------------------------------------------
      Print #plngFileNum, "Record Number: " & plngRecNum
      Print #plngFileNum, "Time Written: " & pvntTimeWritten
      Print #plngFileNum, "SourceName: " & pstrSourceName
      If pintEventCategory = 0 Then
         Print #plngFileNum, "Event Category: NONE"
      Else
         Print #plngFileNum, "Event Category: " & pintEventCategory
      End If
      Print #plngFileNum, "Event ID: " & plngEventID
      If plngSidLength = 0 Then
         Print #plngFileNum, "User: N/A"
      Else
         pstrTmp = Mid$(pstrBuffer, plngSidOffset, plngSidLength)
         pstrSid = ""
         For k% = 1 To plngSidLength
            pstrSid = pstrSid & Format$(Hex$(Asc(Mid$(pstrTmp, k%, 1))), "00") & " "
         Next k%
         Print #plngFileNum, "User: " & pstrSid
      End If
      Print #plngFileNum, "ComputerName: " & pstrComputerName
      Print #plngFileNum, "Event Type: " & pstrEventType
      Print #plngFileNum, "Description: " & vbCrLf & pstrMsgString
      If plngDataLength > 0 Then
         pstrData = Mid$(pstrBuffer, plngDataOffset + 1, plngDataLength)
         Print #plngFileNum, "Data: ";
         For k% = 1 To plngDataLength
            Print #plngFileNum, Format$(Hex$(Asc(Mid$(pstrData, k%, 1))), "00") & " ";
         Next k%
         Print #plngFileNum, ""
      Else
         Print #plngFileNum, "Data: N/A"
      End If
     
      Print #plngFileNum, ""
   Next pintLoop%
   
   
   ' ----------------------------------------------
   '
   ' ----------------------------------------------
   Close #plngFileNum
   
End Sub


' -------------------------------------------------
'
' -------------------------------------------------
Private Function GetTimeBias() As Long
   Dim TimeZone      As TIME_ZONE_INFORMATION
   Dim plngRtn       As Long
   Dim plngMinutes   As Long
   
   plngRtn = GetTimeZoneInformation(TimeZone)
   plngMinutes = TimeZone.Bias
   GetTimeBias = plngMinutes * 60 ' For seconds
   
End Function

' -------------------------------------------------
'
' -------------------------------------------------
Private Function GetRegistryEntry(xstrLogType As String, xstrSourceName As String) As String
   Dim plngKeyType      As Long
   Dim plngRtn          As Long
   Dim plngOpenedKey    As Long
   Dim plngLenRtnData   As Long
   Dim pstrRtnString    As String
   Dim pstrValue        As String
   Dim pstrErrorStr     As String
   Dim pstrKey          As String
 
 
   ' -----------------------------------------------------
   ' Turn off errors
   ' -----------------------------------------------------
   On Error Resume Next


   ' --------------------------------------------------
   ' Open key
   ' --------------------------------------------------
   pstrKey = "SYSTEM\CurrentControlSet\Services\EventLog\" _
              & xstrLogType & "\" & xstrSourceName
   plngRtn = RegOpenKeyEx(HKEY_LOCAL_MACHINE, pstrKey, 0&, KEY_READ, _
         plngOpenedKey)
   If plngRtn <> ERROR_SUCCESS Then
      MsgBox "Error OPENING registry key: " & Err.LastDllError
   End If


   ' --------------------------------------------------
   ' Set up buffer for data to be returned in.
   ' --------------------------------------------------
   plngLenRtnData = 255
   pstrRtnString = Space$(plngLenRtnData)


   ' --------------------------------------------------
   ' Read key -- try again if buffer isn't big enough
   ' --------------------------------------------------
ValueTryAgain:
   plngRtn = RegQueryValueEx(plngOpenedKey, "EventMessageFile", ByVal 0&, _
         plngKeyType, ByVal pstrRtnString, plngLenRtnData)

   If plngRtn = ERROR_MORE_DATA Then
      pstrRtnString = Space$(plngLenRtnData)
      GoTo ValueTryAgain
   ElseIf plngRtn <> ERROR_SUCCESS Then
      MsgBox "Error READING registry value: " & Err.LastDllError
   Else
      GetRegistryEntry = Left$(pstrRtnString, plngLenRtnData - 1)
   End If


   ' --------------------------------------------------
   ' Always close opened keys!
   ' --------------------------------------------------
   plngRtn = RegCloseKey(plngOpenedKey)

   On Error GoTo 0

End Function

' -------------------------------------------------
'
' -------------------------------------------------
Private Sub ReplaceParameters(xintNumStrings As Integer, xstrMsgString As String, xstrMsgArg() As String)
   Dim pstrTmpMsgString    As String
   Dim pintLoop            As Integer
   Dim plngPos             As Long
   Dim pstrFirst           As String
   Dim pstrLast            As String
   
   
   pstrTmpMsgString = xstrMsgString
   On Error Resume Next
   For pintLoop = 1 To xintNumStrings
      plngPos = InStr(pstrTmpMsgString, "%" & CStr(pintLoop))
      If plngPos > 0 Then
         pstrFirst = Mid$(pstrTmpMsgString, 1, plngPos - 1)
         pstrLast = Mid$(pstrTmpMsgString, plngPos + 2)
         pstrTmpMsgString = pstrFirst & xstrMsgArg(pintLoop) & pstrLast
      End If
   Next pintLoop
   
   If Right$(pstrTmpMsgString, 2) = vbCrLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 2)
   ElseIf Right$(pstrTmpMsgString, 1) = vbLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   ElseIf Right$(pstrTmpMsgString, 1) = vbCr Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   End If
   
   xstrMsgString = pstrTmpMsgString

End Sub

' -------------------------------------------------
'
' -------------------------------------------------
Private Function ClearLastCrLf(xstrTmpMsgString As String) As String
   Dim pstrTmpMsgString    As String
   
   pstrTmpMsgString = xstrTmpMsgString
   
   If Right$(pstrTmpMsgString, 2) = vbCrLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 2)
   ElseIf Right$(pstrTmpMsgString, 1) = vbLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   ElseIf Right$(pstrTmpMsgString, 1) = vbCr Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   End If

   ClearLastCrLf = pstrTmpMsgString
   
End Function

' -------------------------------------------------
' Note:  Passing strings to function, the trailing
'        Chr$(0)'s will be truncated -- need to add
'        them back
' -------------------------------------------------
Private Function CVI(xstrNumber As String) As Integer
   Dim pintTmp    As Integer
   
   Select Case Len(Trim(xstrNumber))
      Case 0
         Debug.Print "Illegal function call"
         Exit Function
      Case 1
         xstrNumber = xstrNumber & Chr$(0)
      Case Is > 2
         Debug.Print "Illegal function call"
         Exit Function
   End Select
   
   CopyMem pintTmp, ByVal xstrNumber, 2
   CVI = pintTmp
   
End Function

' -------------------------------------------------
' Note:  Passing strings to function, the trailing
'        Chr$(0)'s will be truncated -- need to add
'        them back
' Also, in one case, we will be passing only 2
'        characters to the function in order to get
'        the low-order number for EventID (so it
'        will be the same as shown in the Event
'        Viewer)
' -------------------------------------------------
Private Function CVL(xstrNumber As String) As Long
   Dim plngTmp    As Long
   
   Select Case Len(Trim(xstrNumber))
      Case 0
         Debug.Print "Illegal function call"
         Exit Function
      Case 1
         xstrNumber = xstrNumber & Chr$(0) & Chr$(0) & Chr$(0)
      Case 2
         xstrNumber = xstrNumber & Chr$(0) & Chr$(0)
      Case 3
         xstrNumber = xstrNumber & Chr$(0)
      Case Is > 4
         Debug.Print "Illegal function call"
         Exit Function
   End Select
   
   CopyMem plngTmp, ByVal xstrNumber, 4
   CVL = plngTmp
   
End Function
