VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "EventLogs"
Attribute VB_Creatable = True
Attribute VB_Exposed = True
' *************************************************
' *        Copyright (C) L.J. Johnson, 1995       *
' *               All Rights Reserved.            *
' *************************************************

Option Explicit

' -------------------------------------------------
' Type of item to look up in the registry
' -------------------------------------------------
Private Const EventMessage = 0
Private Const ParameterMessage = 1


' -------------------------------------------------
' Save the filter and filter type
' -------------------------------------------------
Private mlngFilterType           As Long
Private mvntFilter               As Variant
Private mlngEventReadLogForward  As Long
Private mlngEventDataReturnHex   As Long


' -------------------------------------------------
' Share the module-level event log handle
' -------------------------------------------------
Private mlngEventLogHwd    As Long


' -------------------------------------------------
' Not an API declaration -- this is the UDT that
'   stores the info on each event record
' -------------------------------------------------
Private Type EventRecord
   EventTimeWritten  As Variant
   EventSourceName   As String
   EventUserName     As String
   EventComputerName As String
   EventType         As String
   EventDescription  As String
   EventData         As String
   EventCategory     As Integer
   EventRecordNum    As Long
   EventID           As Long
End Type
Private colEventRecord() As EventRecord
 

' -------------------------------------------------
' Used as part of error string return
' -------------------------------------------------
Private Const EVENT_SOURCENAME = "ReadEventLogs.EventLogs."
Private Const FORMAT_SOURCENAME = "ReadEventLogs.FormatMsg."


' -------------------------------------------------
' Public -- keep track of our own error info
' -------------------------------------------------
Public LastEventErrorNumber      As Long
Public LastEventErrorSource      As String
Public LastEventErrorDescription As String


' -------------------------------------------------
' Used to pass info within the class module
' -------------------------------------------------
Private mstrTypeEventLog   As String
Private mlngCount          As Long


' -------------------------------------------------
' Used to get time/date info from event records
' -------------------------------------------------
Private Type SYSTEMTIME
   wYear          As Integer
   wMonth         As Integer
   wDayOfWeek     As Integer
   wDay           As Integer
   wHour          As Integer
   wMinute        As Integer
   wSecond        As Integer
   wMilliseconds  As Integer
End Type

Private Type TIME_ZONE_INFORMATION
   Bias              As Long
   StandardName      As String * 32
   StandardDate      As SYSTEMTIME
   StandardBias      As Long
   DaylightName      As String * 32
   DaylightDate      As SYSTEMTIME
   DaylightBias      As Long
End Type


' -------------------------------------------------
' 32-bit API declares
' -------------------------------------------------
Private Declare Function OpenEventLog Lib "advapi32" _
   Alias "OpenEventLogA" (ByVal lpUNCServerName As String, _
   ByVal lpEventSourceName As String) As Long
Private Declare Function GetNumberOfEventLogRecords _
   Lib "advapi32.dll" (ByVal hEventLog As Long, _
   NumberOfRecords As Long) As Long
Private Declare Function ReadEventLog Lib "advapi32.dll" _
   Alias "ReadEventLogA" (ByVal hEventLog As Long, _
   ByVal dwReadFlags As Long, ByVal dwRecordOffset As Long, _
   lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, _
   pnBytesRead As Long, pnMinNumberOfBytesNeeded As Long) As Long
Private Declare Function LoadLibrary Lib "kernel32" _
   Alias "LoadLibraryA" (ByVal lpLibFileName As String) As Long
Private Declare Function FreeLibrary _
   Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function GetTimeZoneInformation _
   Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
Private Declare Function ExpandEnvironmentStrings Lib "kernel32" _
   Alias "ExpandEnvironmentStringsA" (ByVal lpSrc As String, _
   ByVal lpDst As String, ByVal nSize As Long) As Long


' -------------------------------------------------
' Defines for the READ flags for Eventlogging
' -------------------------------------------------
Private Const EVENTLOG_SEQUENTIAL_READ = &H1
Private Const EVENTLOG_SEEK_READ = &H2
Private Const EVENTLOG_FORWARDS_READ = &H4
Private Const EVENTLOG_BACKWARDS_READ = &H8



' -------------------------------------------------
' Return the setting for the EventReadLogForward
'   property -- either TRUE or FALSE
' Note: if you don't set it explicitly, it defaults
'   to false
' -------------------------------------------------
Public Property Get EventReadLogForward() As Long
Attribute EventReadLogForward.VB_HelpID = 11
Attribute EventReadLogForward.VB_Description = "If you dont set the EventReadLogForward property, the server will read the event log items backwards (latest date first), which is the same as the default for the built-in NT Event Viewer program"
   EventReadLogForward = mlngEventReadLogForward
End Property

' -------------------------------------------------
' Wrapper for the GetTimeZoneInformation API call,
'   but it also multipies the results (in minutes)
'   by 60 to get the value in seconds
' -------------------------------------------------
Private Function GetTimeBias() As Long
   Dim TimeZone      As TIME_ZONE_INFORMATION
   Dim plngRtn       As Long
   Dim plngMinutes   As Long
   
   plngRtn = GetTimeZoneInformation(TimeZone)
   plngMinutes = TimeZone.Bias
   GetTimeBias = plngMinutes * 60 ' For seconds
   
End Function


' -------------------------------------------------
' Returns the EventLogType property
' -------------------------------------------------
Public Property Get EventTypeLog() As String
Attribute EventTypeLog.VB_HelpID = 3
Attribute EventTypeLog.VB_Description = "You must set the type of event log via the EventTypeLog property before calling the ReadEventEntries method"
   EventTypeLog = mstrTypeEventLog
End Property

' -------------------------------------------------
' Set the type of event log -- traps for incorrect
'   values from calling program, and sets the 3
'   error information variables if incorrect.
' -------------------------------------------------
Public Property Let EventTypeLog(xstrTypeEventLog As String)
   
   Select Case xstrTypeEventLog
      Case EVNT_SYSTEM, EVNT_APP, EVNT_SECURITY
         mstrTypeEventLog = xstrTypeEventLog
      Case Else
         LastEventErrorNumber = (ERR_FAILED_SET_LOG_TYPE + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "LogType"
         LastEventErrorDescription = "Error setting the LogType: Passed: " & _
                              xstrTypeEventLog & ".  LogType must be 'Application'," _
                              & " 'Security', or 'System'."
         mstrTypeEventLog = CStr(&HFFFFFFFF)
   End Select
   
End Property





' -------------------------------------------------
' This is the heart of the OLE server.  It reads
'   the selected event log on the selected NT
'   server, and makes calls to all the private
'   properties and functions
' -------------------------------------------------
Public Function ReadEventEntries() As Long
Attribute ReadEventEntries.VB_HelpID = 2
Attribute ReadEventEntries.VB_Description = "After opening an Event Log, the ReadEventEntries will read either all or just the filtered entries for that log"
   Dim plngNumUnfilteredRecords  As Long  ' Count of unfiltered records
   Dim plngIsMatch            As Long     ' Is it a match for the filter?
   Dim plngNumFilteredRecords As Long     ' Count of filtered records
   Dim plngRecCount           As Long     ' Count of filtered records (could = total records if not filtered)
   Dim pstrSid                As String   ' Hex string of Security Identifier
   Dim pbyteBuffer()          As Byte     ' Used to read the event log records
   Dim pstrMultiRecBuffer     As String   ' One or more event log record(s)
   Dim pstrBuffer             As String   ' A single event log record
   Dim plngEventLogHwd        As Long     ' Handle to the current open event log
   Dim plngReadFlags          As Long     ' Flags used to read the event log
   Dim plngReadRecordOffset   As Long     ' Offset for the records read in event log
   Dim plngNumBytesToRead     As Long     ' Number of bytes TO read from event log
   Dim plngNumBytesRead       As Long     ' Number of bytes actually READ from event log
   Dim plngNumRecords         As Long     ' Total number of records in this event log
   Dim plngMinNumBytesNeeded  As Long     ' Used in ReadEventLog API call
   Dim pvntStartDate          As Variant  ' Used to store the "01/01/1970 00:00:00" date
   Dim pvntAddDate            As Variant  ' Date returned from the event log record
   Dim plngBiasTimeSecs       As Long     ' Used to adjust time to local time
   Dim plngRtn                As Long     ' Return value from ReadEventLog API call
   Dim plngPos                As Long     ' Position within a string
   Dim pstrMsgString          As String   ' Message string from the event log record
   Dim pintInnerLoop          As Integer  ' Loop when changing a binary string to hex
   
   ' Used to get info from returned string
   Dim plngRecLen             As Long
   Dim plngRecNum             As Long
   Dim pvntTimeGenerated      As Variant
   Dim pvntEventTimeWritten   As Variant
   Dim plngEventID            As Long
   Dim plngEventIdForFile     As Long
   Dim pintEventType          As Integer
   Dim pintNumStrings         As Integer
   Dim pintEventCategory      As Integer
   Dim plngClosingRecNumber   As Long
   Dim plngStringOffset       As Long
   Dim plngSidLength          As Long
   Dim plngSidOffset          As Long
   Dim plngDataLength         As Long
   Dim plngDataOffset         As Long
   Dim pstrEventType          As String
   Dim pstrTmp                As String
   Dim pstrEventSourceName    As String
   Dim pstrEventComputerName  As String
   Dim pstrStrings            As String
   Dim pstrData               As String
   
   
   ' ----------------------------------------------
   ' Turn on error trapping -- this is an OLE
   '   server
   ' ----------------------------------------------
   On Error Resume Next
   
   
   ' ----------------------------------------------
   ' Make sure EventType has been set
   ' ----------------------------------------------
   If Len(Trim$(mstrTypeEventLog)) = 0 Then
      ' -------------------------------------------
      ' Already have error information set from the
      '   call to the property -- just exit
      ' -------------------------------------------
      ReadEventEntries = False
      mlngCount = 0
      Exit Function
   End If
   
   
   ' ----------------------------------------------
   ' Set the starting date and bias (ie, Time Zone)
   '   to figure the dates
   ' ----------------------------------------------
   pvntStartDate = CDate("01/01/1970 00:00:00")
   plngBiasTimeSecs = GetTimeBias()
   
   
   ' ----------------------------------------------
   ' Make sure that the event log has been opened
   ' ----------------------------------------------
   If mlngEventLogHwd = &HFFFFFFFF Then
      ' -------------------------------------------
      ' Has never been set -- see Initialize event
      ' -------------------------------------------
      LastEventErrorNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      LastEventErrorSource = EVENT_SOURCENAME & "ReadEventEntries"
      LastEventErrorDescription = "The Event Log has not been opened yet.  Use OpenAnyEventLog property."
      ReadEventEntries = False
      mlngCount = 0
      Exit Function
   ElseIf mlngEventLogHwd <> 0 Then
      plngEventLogHwd = mlngEventLogHwd
   Else
      ' -------------------------------------------
      ' Error info set when set the Server Name
      ' -------------------------------------------
      ReadEventEntries = False
      mlngCount = 0
      Exit Function
   End If
   
   
   ' ----------------------------------------------
   ' See how many event records are in this log
   ' ----------------------------------------------
   If API_NumberOfEventLogRecords(plngEventLogHwd) = True Then
      plngNumRecords = CountEventRecords
   End If
   
   If plngNumRecords <= 0 Then
      ' -------------------------------------------
      ' No records -- set number of records to zero
      '   and exit the function
      ' -------------------------------------------
      ReadEventEntries = False
      mlngCount = 0
      Exit Function
   Else
      ' -------------------------------------------
      ' Redimension the type array to total number
      '   of records
      ' -------------------------------------------
      ReDim colEventRecord(1 To plngNumRecords) As EventRecord
   End If
   
   
   ' ----------------------------------------------
   ' Set the read flags
   ' ----------------------------------------------
   If mlngEventReadLogForward = True Then
      plngReadFlags = EVENTLOG_SEQUENTIAL_READ Or _
                      EVENTLOG_FORWARDS_READ
   Else
      plngReadFlags = EVENTLOG_SEQUENTIAL_READ Or _
                      EVENTLOG_BACKWARDS_READ
   End If
   
   
   ' ----------------------------------------------
   ' Initialize some variables
   ' ----------------------------------------------
   ReadEventEntries = True
   plngNumFilteredRecords = 0
   plngNumUnfilteredRecords = 0
   plngReadRecordOffset = 1
   
   
   ' ----------------------------------------------
   ' Loop thru the record numbers
   ' ----------------------------------------------
   Do While plngNumUnfilteredRecords < plngNumRecords
   
   
      ' -------------------------------------------
      ' 1024 bytes should over most single event
      '   log records.
      ' Note that incrementing the size of the
      '   array won't necessarily speed up the
      '   procedure -- experiment
      ' -------------------------------------------
      plngNumBytesToRead = 1024 * (plngNumRecords - plngNumUnfilteredRecords)
      If plngNumBytesToRead > 16384 Then
         plngNumBytesToRead = 16384
      ElseIf plngNumBytesToRead < 4096 Then
         plngNumBytesToRead = 4096
      End If
      
TryAgain:
      ReDim pbyteBuffer(0 To plngNumBytesToRead - 1)

   
      ' -------------------------------------------
      ' Read the event log (multiple records) and
      '   check for errors
      ' -------------------------------------------
      plngRtn = ReadEventLog(plngEventLogHwd, _
         plngReadFlags, plngReadRecordOffset, _
         pbyteBuffer(0), plngNumBytesToRead, _
         plngNumBytesRead, plngMinNumBytesNeeded)
   
      If plngRtn = False Then
         If Err.LastDllError = ERROR_HANDLE_EOF Then
            ' -------------------------------------
            ' End of the records
            ' -------------------------------------
            Exit Do
         ElseIf Err.LastDllError = ERROR_INSUFFICIENT_BUFFER Then
            ' -------------------------------------
            ' OK, 16k wasn't big enough -- set to
            '   the value returned by the function
            '   for the number of bytes needed
            ' -------------------------------------
            plngNumBytesToRead = plngMinNumBytesNeeded
            GoTo TryAgain
         Else
            LastEventErrorNumber = (ERR_READING_EVENT_LOG + vbObjectError)
            LastEventErrorSource = EVENT_SOURCENAME & "ReadEventEntries"
            LastEventErrorDescription = ReturnApiErrString(Err.LastDllError)
            ReadEventEntries = False
            mlngCount = 0
            Exit Function
         End If
      End If
   

      ' -------------------------------------------
      ' "Trim" the byte buffer to the size
      '   actually read
      ' -------------------------------------------
      ReDim Preserve pbyteBuffer(0 To plngNumBytesRead - 1)
   
   
      ' -------------------------------------------
      ' Copy the memory to a string then clear the
      '   byte array
      ' -------------------------------------------
      pstrMultiRecBuffer = Space$(plngNumBytesRead)
      CopyMem ByVal pstrMultiRecBuffer, pbyteBuffer(0), plngNumBytesRead
      Erase pbyteBuffer
   
   
      ' -------------------------------------------
      ' Loop for each record in the buffer
      ' -------------------------------------------
      Do While Len(pstrMultiRecBuffer) > 0
   
         ' ----------------------------------------
         ' Don't exceed the number of records in
         '   the event log
         ' ----------------------------------------
         If plngNumUnfilteredRecords > plngNumRecords Then
            Exit Do
         End If
   
   
         ' ----------------------------------------
         ' Increment the number of records
         ' ----------------------------------------
         plngNumUnfilteredRecords = plngNumUnfilteredRecords + 1
   
   
         ' ----------------------------------------
         ' Get the length of the next record and
         '   set the buffer to that length
         ' Delete that record from the combined-
         '   records string
         ' ----------------------------------------
         plngRecLen = CVL(Mid$(pstrMultiRecBuffer, 1, 4))
         pstrBuffer = Mid$(pstrMultiRecBuffer, 1, plngRecLen)
         pstrMultiRecBuffer = Mid$(pstrMultiRecBuffer, plngRecLen + 1)
   
   
         ' ----------------------------------------
         ' Get the raw info from the event log
         ' ----------------------------------------
         plngRecNum = CVL(Mid$(pstrBuffer, 9, 4))
         pvntAddDate = CVL(Mid$(pstrBuffer, 13, 4))
         pvntTimeGenerated = DateAdd("s", CLng(pvntAddDate) - plngBiasTimeSecs, pvntStartDate)
         pvntAddDate = CVL(Mid$(pstrBuffer, 17, 4))
         pvntEventTimeWritten = DateAdd("s", CLng(pvntAddDate) - plngBiasTimeSecs, pvntStartDate)
         plngEventID = CVL(Mid$(pstrBuffer, 21, 2))
         plngEventIdForFile = CVL(Mid$(pstrBuffer, 21, 4))
         pintEventType = CVI(Mid$(pstrBuffer, 25, 2))
         pintNumStrings = CVI(Mid$(pstrBuffer, 27, 2))
         pintEventCategory = CVI(Mid$(pstrBuffer, 29, 2))
         plngStringOffset = CVL(Mid$(pstrBuffer, 37, 4))
         plngSidLength = CVL(Mid$(pstrBuffer, 41, 4))
         plngSidOffset = CVL(Mid$(pstrBuffer, 45, 4))
         plngDataLength = CVL(Mid$(pstrBuffer, 49, 4))
         plngDataOffset = CVL(Mid$(pstrBuffer, 53, 4))
   
   
         ' ----------------------------------------
         ' Get the EventSourceName and
         '   EventComputerName strings
         ' ----------------------------------------
         pstrTmp = Mid$(pstrBuffer, 57)
         On Error Resume Next
         plngPos = InStr(pstrTmp, Chr$(0))
         If plngPos > 0 Then
            pstrEventSourceName = Mid$(pstrTmp, 1, plngPos - 1)
         End If
   
         If plngPos > 0 Then
            pstrTmp = Mid$(pstrTmp, plngPos + 1)
            plngPos = InStr(pstrTmp, Chr$(0))
            If plngPos > 0 Then
               pstrEventComputerName = Mid$(pstrTmp, 1, plngPos - 1)
            End If
         End If
   
   
         ' ----------------------------------------
         ' Get the EventType
         ' ----------------------------------------
         Select Case CLng(pintEventType)
            Case EVENTLOG_SUCCESS
               pstrEventType = "Success"
            Case EVENTLOG_ERROR_TYPE
               pstrEventType = "Error"
            Case EVENTLOG_WARNING_TYPE
               pstrEventType = "Warning"
            Case EVENTLOG_INFORMATION_TYPE
               pstrEventType = "Information"
            Case EVENTLOG_AUDIT_SUCCESS
               pstrEventType = "Audit Success"
            Case EVENTLOG_AUDIT_FAILURE
               pstrEventType = "Audit Failure"
         End Select
   
   
         ' ----------------------------------------
         ' For no filter, all records match
         ' ----------------------------------------
         If mlngFilterType = Filter_Type_None Then
            plngIsMatch = True
   
   
         ' ----------------------------------------
         ' Have a filter -- this record may or may
         '   not match
         ' ----------------------------------------
         Else
            plngIsMatch = False
   
            Select Case mlngFilterType
               Case Filter_Type_TimeBefore
                  If CDate(pvntEventTimeWritten) < CDate(mvntFilter) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_TimeAfter
                  If CDate(pvntEventTimeWritten) >= CDate(mvntFilter) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_EventType
                  If CLng(pintEventType) = CLng(mvntFilter) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_Source
                  If UCase$(pstrEventSourceName) = UCase$(CStr(mvntFilter)) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_Category
                  If CInt(pintEventCategory) = CInt(mvntFilter) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_Computer
                  If UCase$(pstrEventComputerName) = UCase$(CStr(mvntFilter)) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
               Case Filter_Type_EventID
                  If CLng(plngEventID) = CLng(mvntFilter) Then
                     plngNumFilteredRecords = plngNumFilteredRecords + 1
                     plngIsMatch = True
                  End If
            End Select
   
         End If
         
         
         ' ----------------------------------------
         ' Get the message string ONLY IF IT IS
         '   going to be returned.  This will save
         '   a LOT of time for filtered records
         ' ----------------------------------------
         If plngIsMatch = True Then
            If pintNumStrings > 0 Then
               pstrStrings = Mid$(pstrBuffer, plngStringOffset + 1, (plngDataOffset - plngStringOffset))
            End If
            If Len(Trim$(pstrEventSourceName)) = 0 Then
               pstrMsgString = "The description for Event ID ("
               pstrMsgString = pstrMsgString & CStr(plngEventID)
               pstrMsgString = pstrMsgString & ") in Source ("
               pstrMsgString = pstrMsgString & pstrEventSourceName
               pstrMsgString = pstrMsgString & " ) could not be found. "
               pstrMsgString = pstrMsgString & "It contains the following "
               pstrMsgString = pstrMsgString & "inserting strings: "
               pstrMsgString = pstrMsgString & pstrStrings
            Else
               pstrMsgString = ResourceString(pstrEventSourceName, _
                                    pintNumStrings, pstrStrings, _
                                    plngEventIdForFile, plngEventID)
            End If
         End If
         
         
         ' ----------------------------------------
         ' Only add to array if this is a matching
         '   record (with no filter, all records
         '   match)
         ' ----------------------------------------
         If plngIsMatch = True Then
            If mlngFilterType = Filter_Type_None Then
               plngRecCount = plngNumUnfilteredRecords
            Else
               plngRecCount = plngNumFilteredRecords
            End If

            colEventRecord(plngRecCount).EventRecordNum = plngRecNum
            colEventRecord(plngRecCount).EventTimeWritten = pvntEventTimeWritten
            colEventRecord(plngRecCount).EventSourceName = pstrEventSourceName
            colEventRecord(plngRecCount).EventCategory = pintEventCategory
            colEventRecord(plngRecCount).EventID = plngEventID
            colEventRecord(plngRecCount).EventComputerName = pstrEventComputerName
            colEventRecord(plngRecCount).EventType = pstrEventType
            colEventRecord(plngRecCount).EventDescription = pstrMsgString

   
            ' -------------------------------------
            ' If have a SID, convert to hex
            ' -------------------------------------
            If plngSidLength = 0 Then
               colEventRecord(plngRecCount).EventUserName = "N/A"
            Else
               pstrTmp = Mid$(pstrBuffer, plngSidOffset, plngSidLength)
               pstrSid = ""
               For pintInnerLoop = 1 To plngSidLength
                  pstrSid = pstrSid & Format$(Hex$(Asc(Mid$(pstrTmp, pintInnerLoop, 1))), "00") & " "
               Next pintInnerLoop
               colEventRecord(plngRecCount).EventUserName = pstrSid
            End If
   
   
            ' -------------------------------------
            ' If have any data, convert to hex
            ' -------------------------------------
            If plngDataLength > 0 Then
               pstrTmp = Mid$(pstrBuffer, plngDataOffset + 1, plngDataLength)
               pstrData = ""
               ' ----------------------------------
               ' Convert to hex only if less than
               '   256 bytes or if the user wants
               '   it that way
               ' ----------------------------------
               If mlngEventDataReturnHex = False Then
                  If Len(pstrTmp) > 256 Then
                     pstrData = pstrTmp
                  Else
                     For pintInnerLoop = 1 To plngDataLength
                        pstrData = pstrData & Format$(Hex$(Asc(Mid$(pstrTmp, pintInnerLoop, 1))), "00") & " "
                     Next pintInnerLoop
                  End If
               Else
                  For pintInnerLoop = 1 To plngDataLength
                     pstrData = pstrData & Format$(Hex$(Asc(Mid$(pstrTmp, pintInnerLoop, 1))), "00") & " "
                  Next pintInnerLoop
               End If
               colEventRecord(plngRecCount).EventData = pstrData
            Else
               colEventRecord(plngRecCount).EventData = "N/A"
            End If
   
         End If
      
      Loop
   Loop
   
   
   ' ----------------------------------------------
   ' Re-adjust the number of items in array if it
   '   was filtered, and set the number of records
   '   in the module-level variable
   ' ----------------------------------------------
   If mlngFilterType <> Filter_Type_None Then
      ReDim Preserve colEventRecord(1 To plngNumFilteredRecords)
      mlngCount = plngNumFilteredRecords
   End If
   
   
   ' ----------------------------------------------
   ' Turn error trapping off
   ' ----------------------------------------------
   On Error GoTo 0
   
End Function

' -------------------------------------------------
' Property to return the number of event records
' -------------------------------------------------
Public Property Get CountEventRecords() As Long
Attribute CountEventRecords.VB_HelpID = 8
Attribute CountEventRecords.VB_Description = "Count of records in filtered Event Log record"
   CountEventRecords = mlngCount
End Property



' -------------------------------------------------
' Get the number of records in this event log using
'   the API -- note that this is not exposed by
'   the object, since the "actual" number may
'   changed based on the filtering applied
' -------------------------------------------------
Private Function API_NumberOfEventLogRecords(xlngEventLogHwd As Long) As Long
   Dim plngNumOfRecords    As Long
   
   If GetNumberOfEventLogRecords(xlngEventLogHwd, _
                  plngNumOfRecords) <> False Then
      mlngCount = plngNumOfRecords
      API_NumberOfEventLogRecords = True
   Else
      API_NumberOfEventLogRecords = False
   End If

End Function


' -------------------------------------------------
' Return the time/date that the event log record
'   (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventTimeWritten(xlngIndex As Long) As Variant
Attribute EventTimeWritten.VB_HelpID = 20
Attribute EventTimeWritten.VB_Description = "Returns a variant giving the time/date stamp that this Event Log record was written to"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventTimeWritten"
         LastEventErrorDescription = "The passed index number is not valid."
         EventTimeWritten = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventTimeWritten = colEventRecord(xlngIndex).EventTimeWritten

End Property



' -------------------------------------------------
' Return the Event Source Name of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventSourceName(xlngIndex As Long) As String
Attribute EventSourceName.VB_HelpID = 19
Attribute EventSourceName.VB_Description = "Returns a string that contains the name of the source (application, service, driver, subsystem) that generated this log entry"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventSourceName"
         LastEventErrorDescription = "The passed index number is not valid."
         EventSourceName = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventSourceName = colEventRecord(xlngIndex).EventSourceName

End Property



' -------------------------------------------------
' Return the Event User Name of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventUserName(xlngIndex As Long) As String
Attribute EventUserName.VB_HelpID = 22
Attribute EventUserName.VB_Description = "Returns a space-delimited hex string of the SID (Security Identifier) of the active user at the time this event was logged"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventUserName"
         LastEventErrorDescription = "The passed index number is not valid."
         EventUserName = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventUserName = colEventRecord(xlngIndex).EventUserName

End Property



' -------------------------------------------------
' Return the Event Computer Name of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventComputerName(xlngIndex As Long) As String
Attribute EventComputerName.VB_HelpID = 12
Attribute EventComputerName.VB_Description = "String that specifies the name of the computer that generated this event"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventComputerName"
         LastEventErrorDescription = "The passed index number is not valid."
         EventComputerName = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventComputerName = colEventRecord(xlngIndex).EventComputerName

End Property



' -------------------------------------------------
' Return the Event Type of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventType(xlngIndex As Long) As String
Attribute EventType.VB_HelpID = 21
Attribute EventType.VB_Description = "There are only 5 possible Event Types: EVENTLOG_ERROR_TYPE, EVENTLOG_WARNING_TYPE, \r\nEVENTLOG_INFORMATION_TYPE, \r\nEVENTLOG_AUDIT_SUCCESS, \r\nEVENTLOG_AUDIT_FAILURE"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventType"
         LastEventErrorDescription = "The passed index number is not valid."
         EventType = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventType = colEventRecord(xlngIndex).EventType

End Property



' -------------------------------------------------
' Return the Event Description of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventDescription(xlngIndex As Long) As String
Attribute EventDescription.VB_HelpID = 13
Attribute EventDescription.VB_Description = "Returns a text string that describes the event that occurred"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventDescription"
         LastEventErrorDescription = "The passed index number is not valid."
         EventDescription = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventDescription = colEventRecord(xlngIndex).EventDescription

End Property



' -------------------------------------------------
' Return the Event Category of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventCategory(xlngIndex As Long) As Integer
Attribute EventCategory.VB_HelpID = 17
Attribute EventCategory.VB_Description = "One of the properties of an Event Log record (specific to the Source file)"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventCategory"
         LastEventErrorDescription = "The passed index number is not valid."
         EventCategory = 0
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventCategory = colEventRecord(xlngIndex).EventCategory

End Property



' -------------------------------------------------
' Return the Event Record Number of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventRecordNum(xlngIndex As Long) As Long
Attribute EventRecordNum.VB_HelpID = 16
Attribute EventRecordNum.VB_Description = "Returns a long integer giving the internal order number of this record"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventRecordNum"
         LastEventErrorDescription = "The passed index number is not valid."
         EventRecordNum = 0
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventRecordNum = colEventRecord(xlngIndex).EventRecordNum

End Property



' -------------------------------------------------
' Return the Event ID of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventID(xlngIndex As Long) As Long
Attribute EventID.VB_HelpID = 6
Attribute EventID.VB_Description = "Returns a long integer that is specific to the source that generated the Event Log entry"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventID"
         LastEventErrorDescription = "The passed index number is not valid."
         EventID = 0
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventID = colEventRecord(xlngIndex).EventID

End Property



' -------------------------------------------------
' Return the Event DATA of an event log
'   record (based on passed index value)
' Check for zero values or values greater than
'   the number of log entries in the filtered
'   result set.
' -------------------------------------------------
Public Property Get EventData(xlngIndex As Long) As String
Attribute EventData.VB_HelpID = 14
Attribute EventData.VB_Description = "A text string that describes the event that occurred"

   On Error Resume Next
   Select Case xlngIndex
      Case 0, Is > mlngCount
         LastEventErrorNumber = (ERR_BAD_INDEX + vbObjectError)
         LastEventErrorSource = EVENT_SOURCENAME & "EventData"
         LastEventErrorDescription = "The passed index number is not valid."
         EventData = ""
         Exit Property
      Case Else
         LastEventErrorNumber = 0
         LastEventErrorSource = ""
         LastEventErrorDescription = ""
   End Select

   EventData = colEventRecord(xlngIndex).EventData

End Property



' -------------------------------------------
' Read the strings in the resource file
' Note:  This is private, not exposed by
'        server
' -------------------------------------------
Private Function ResourceString(xstrEventSourceName As String, _
               xintNumStrings As Integer, _
               xstrStrings As String, _
               xlngEventIdForFile As Long, _
               xlngEventIdForDisplay As Long) As String
   Dim pRegistryDB         As New RegistryDB
   
   Dim plngNumEventIDs     As Long
   Dim plngNumLibNames     As Long
   Dim pstrLibName         As String
   Dim pstrMsgArg()        As String
   Dim pstrMsgString       As String
   Dim plngRtn             As Long
   Dim plngRtnErr          As Long
   Dim plngPos             As Long
   Dim plngPos2            As Long
   Dim plngModHwnd         As Long
   Dim pintLoop            As Integer
   Dim pintFoundLib        As Integer
   Dim pstrNewLibName      As String
   Dim pstrSource          As String
   Dim ptypMsgs            As FmtMsgArrayType
   Dim pintTransRtn        As Integer
   Dim pstrApiError        As String
   
   Static pstaLibNames()   As Source_Lib_Type
   

   ' ----------------------------------------------
   ' Exit if passed a blank SourceName
   ' ----------------------------------------------
   If Len(Trim$(xstrEventSourceName)) <= 0 Then
      ResourceString = ""
      Exit Function
   End If
   
   
   ' ----------------------------------------------
   ' Get the size of the pstaLibNames array
   ' ----------------------------------------------
   On Error Resume Next
   plngNumLibNames = UBound(pstaLibNames)
   On Error Resume Next
   
   
   ' ----------------------------------------------
   ' See if we already have the libray name in the
   '   static array
   ' ----------------------------------------------
   If plngNumLibNames > 0 Then
      pintFoundLib = False
      For pintLoop = 1 To plngNumLibNames
         If pstaLibNames(pintLoop).SourceName = xstrEventSourceName Then
            pstrLibName = pstaLibNames(pintLoop).LibName
            pintFoundLib = True
            Exit For
         End If
      Next pintLoop
   Else
      pintFoundLib = False
   End If


   ' ----------------------------------------------
   ' Didn't find the libary name in array,
   '   so get it with the API
   ' ----------------------------------------------
   If pintFoundLib = False Then
      
      ' -------------------------------------------
      ' Get info from the registry
      ' -------------------------------------------
      pRegistryDB.LogType = mstrTypeEventLog
      pRegistryDB.SourceName = xstrEventSourceName
      If pRegistryDB.RegistryEntry(EventMessage) = True Then
         pstrLibName = pRegistryDB.RegistryEntryText
      Else
         ' ----------------------------------------
         ' NOTE:  You may want to just return a
         '        blank string, but this is the
         '        same string as is present in the
         '        Event Viewer
         ' ----------------------------------------
         pstrMsgString = "The description for Event ID ("
         pstrMsgString = pstrMsgString & CStr(xlngEventIdForDisplay) & ") "
         pstrMsgString = pstrMsgString & "in Source (" & Trim$(xstrEventSourceName)
         pstrMsgString = pstrMsgString & ") could not be found. It contains"
         pstrMsgString = pstrMsgString & " the following insertions string(s): "
         pstrMsgString = pstrMsgString & xstrStrings
         ResourceString = pstrMsgString
         Exit Function
      End If
   
   
      ' -------------------------------------------
      ' Replace replaceable parameter if there
      ' -------------------------------------------
      pstrLibName = UCase$(pstrLibName)
      plngPos = InStr(pstrLibName, "%")
      If plngPos > 0 Then
         pstrNewLibName = Space$(256)
         plngRtn = ExpandEnvironmentStrings(pstrLibName, pstrNewLibName, Len(pstrNewLibName))
         If plngRtn > 0 Then
            pstrLibName = Mid$(pstrNewLibName, 1, plngRtn)
         End If
      End If
      
      
      ' -------------------------------------------
      ' Add to static array
      ' -------------------------------------------
      plngNumLibNames = plngNumLibNames + 1
      ReDim Preserve pstaLibNames(1 To plngNumLibNames)
      pstaLibNames(plngNumLibNames).SourceName = xstrEventSourceName
      pstaLibNames(plngNumLibNames).LibName = pstrLibName
   End If
   

   ' ----------------------------------------------
   ' If have replacement strings, then parse
   '   them into an array to make them easier
   '   to deal with
   ' ----------------------------------------------
   If xintNumStrings > 0 Then
      ReDim pstrMsgArg(0 To xintNumStrings - 1)
      
      On Error Resume Next
      pintLoop = 0
      Do
         
         plngPos = InStr(xstrStrings, Chr$(0))
         If plngPos > 1 Then
            pstrMsgArg(pintLoop) = Mid$(xstrStrings, 1, plngPos - 1)
            
            
            ' -------------------------------------
            ' Gets more complicated -- we have some
            '   parameters to replace from the
            '   ParameterMessageFile instead of the
            '   EventMessageFile.
            ' -------------------------------------
            If InStr(pstrMsgArg(pintLoop), "%%") > 0 Then
               pstrMsgArg(pintLoop) = ResourceString2(xstrEventSourceName, pstrMsgArg(pintLoop))
            End If
            
            xstrStrings = Mid$(xstrStrings, plngPos + 1)
            pintLoop = pintLoop + 1
         ElseIf plngPos = 1 Then
            ' -------------------------------------
            ' Some have blank 1st parameters
            ' -------------------------------------
            pstrMsgArg(pintLoop) = ""
            xstrStrings = Mid$(xstrStrings, 2)
            pintLoop = pintLoop + 1
         ElseIf xintNumStrings = 1 Then
            pstrMsgArg(pintLoop) = Trim$(xstrStrings)
         End If
      Loop While plngPos > 0
      On Error GoTo 0
   End If


   ' ----------------------------------------------
   ' Get the string into a format which
   '   FormatMessage can use
   ' ----------------------------------------------
   If xintNumStrings > 0 Then
      pintTransRtn = TranslateArray(pstrMsgArg(), ptypMsgs)
   End If
   
   
   ' ----------------------------------------------
   ' Load the module
   ' ----------------------------------------------
   plngModHwnd = LoadLibrary(pstrLibName)

   
   ' ----------------------------------------------
   ' Get the string from the module
   ' ----------------------------------------------
   If plngModHwnd > 0 Then
      plngRtnErr = FormatMsgFromResource(plngModHwnd, xlngEventIdForFile, ptypMsgs, pstrMsgString, pstrApiError)
      If plngRtnErr <> 0 Then
         LastEventErrorNumber = plngRtnErr
         LastEventErrorSource = FORMAT_SOURCENAME & "FormatMsgFromResource"
         LastEventErrorDescription = pstrApiError
      
         pstrMsgString = ""
         ResourceString = ""
         Exit Function
      End If
   End If
   
   
   ' ----------------------------------------------
   ' Clean up the string
   ' ----------------------------------------------
   pstrMsgString = ClearLastCrLf(Trim$(pstrMsgString))
   
   
   ' ----------------------------------------------
   ' Free the module
   ' ----------------------------------------------
   FreeLibrary plngModHwnd
      
      
   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ResourceString = pstrMsgString
   
End Function

' -------------------------------------------------
' Returns the type and actual filter for the
'   EventFilter property
' -------------------------------------------------
Public Property Get EventFilter(xlngType As Long) As Variant
Attribute EventFilter.VB_HelpID = 30
Attribute EventFilter.VB_Description = "If you do not set the EventFilter property, then all event records for a given log are read"
   xlngType = mlngFilterType
   EventFilter = mvntFilter
End Property

' -------------------------------------------------
' Set the filter for the event log -- if invalid,
'   set to no filter (get back all the event log
'   records
' -------------------------------------------------
Public Property Let EventFilter(xlngType As Long, xvntFilter As Variant)
   
   Select Case CLng(xlngType)
      Case 0
         ' ----------------------------------------
         ' No filter
         ' ----------------------------------------
         mlngFilterType = 0
         mvntFilter = ""
      Case Filter_Type_TimeBefore, Filter_Type_TimeAfter
         ' ----------------------------------------
         ' Passed a date/time parameter
         ' ----------------------------------------
         If IsDate(xvntFilter) = True Then
            mlngFilterType = xlngType
            mvntFilter = xvntFilter
         Else
            ' -------------------------------------
            ' Passed incorrect date format -- set
            '   to no filter
            ' -------------------------------------
            mlngFilterType = 0
            mvntFilter = ""
         End If
      Case Filter_Type_EventType
         ' ----------------------------------------
         ' Passed an EventType parameter -- make
         '   sure it is a valid parameter
         ' If not valid, set to no filter
         ' ----------------------------------------
         Select Case xvntFilter
            Case EVENTLOG_ERROR_TYPE, _
                    EVENTLOG_WARNING_TYPE, _
                    EVENTLOG_INFORMATION_TYPE, _
                    EVENTLOG_AUDIT_SUCCESS, _
                    EVENTLOG_AUDIT_FAILURE
               mlngFilterType = xlngType
               mvntFilter = xvntFilter
            Case Else
               ' ----------------------------------
               ' Passed incorrect event type -- set
               '   to no filter
               ' ----------------------------------
               mlngFilterType = 0
               mvntFilter = ""
         End Select
      Case Filter_Type_Source, Filter_Type_Computer
         ' ----------------------------------------
         ' Passed a Source or Computer Name (text
         '   string)
         ' ----------------------------------------
         mlngFilterType = xlngType
         mvntFilter = xvntFilter
      Case Filter_Type_Category
         ' ----------------------------------------
         ' Passed a Category (Integer)
         ' ----------------------------------------
         mlngFilterType = xlngType
         mvntFilter = xvntFilter
      Case Filter_Type_EventID
         ' ----------------------------------------
         ' Passed an EventID (Long)
         ' ----------------------------------------
         mlngFilterType = xlngType
         mvntFilter = xvntFilter
      Case Else
         ' ----------------------------------------
         ' Passed incorrect filter type -- set to
         '   no filter
         ' ----------------------------------------
         mlngFilterType = 0
         mvntFilter = ""
   End Select
         
End Property

 
' -------------------------------------------------
' This method must be called before the
'   ReadEventEntries method.
' It opens the event log type (previously set via
'   the EventTypeLog property) on the NT computer
'   passed to the method
' -------------------------------------------------
Public Function OpenAnyEventLog(xstrServerName As String) As Long
Attribute OpenAnyEventLog.VB_HelpID = 1
Attribute OpenAnyEventLog.VB_Description = "Before reading the Event Log entries, you must first open one of the 3 event logs (Security, System, or Application)"
   Dim plngEventLogHwd     As Long
   
   
   ' ----------------------------------------------
   ' Make sure that the Computer Name is trimmed
   '   and in UNC format
   ' ----------------------------------------------
   xstrServerName = Trim$(xstrServerName)
   If Len(xstrServerName) = 0 Then
      ' -------------------------------------------
      ' Do nothing -- can't put '\\' in front of
      '   a blank string
      ' -------------------------------------------
   ElseIf Left$(xstrServerName, 2) = "\\" Then
      ' -------------------------------------------
      ' Do nothing -- already has '\\' in front
      '   of server of UNC convention
      ' -------------------------------------------
   ElseIf Left$(xstrServerName, 1) = "\" Then
      xstrServerName = "\" & xstrServerName
   Else
      xstrServerName = "\\" & xstrServerName
   End If
   
   
   ' ----------------------------------------------
   ' Default the return value to TRUE
   ' ----------------------------------------------
   OpenAnyEventLog = True
   
   
   ' ----------------------------------------------
   ' See if have an invalid EventLog Type
   ' ----------------------------------------------
   If Len(mstrTypeEventLog) = 0 Then
      mlngEventLogHwd = 0
      OpenAnyEventLog = False
      
      LastEventErrorNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      LastEventErrorSource = EVENT_SOURCENAME & "OpenAnyEventLog"
      LastEventErrorDescription = "Error opening the event log: " & _
                  "EventType not set -- must be 'Application', " & _
                  "'Security', or 'System'."
      Exit Function
   ElseIf CStr(mstrTypeEventLog) = CStr(&HFFFFFFFF) Then
      ' -------------------------------------------
      ' Already set the error, so just exit
      ' -------------------------------------------
      mlngEventLogHwd = 0
      OpenAnyEventLog = False
      Exit Function
   End If
   
   
   ' ----------------------------------------------
   ' Open the event log
   ' ----------------------------------------------
   plngEventLogHwd = OpenEventLog(xstrServerName, mstrTypeEventLog)
   
   
   ' ----------------------------------------------
   ' Check for any errors
   ' ----------------------------------------------
   If plngEventLogHwd = 0 Then
      mlngEventLogHwd = 0
      OpenAnyEventLog = False
      
      LastEventErrorNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      LastEventErrorSource = EVENT_SOURCENAME & "OpenAnyEventLog"
      LastEventErrorDescription = "Error opening the event log, " _
            & mstrTypeEventLog & ", on the Server, " & xstrServerName _
            & ": " & ReturnApiErrString(Err.LastDllError)
      Exit Function
   Else
      mlngEventLogHwd = plngEventLogHwd
   End If

End Function

Private Sub Class_Initialize()
   ' ----------------------------------------------
   ' Indicates that an event log has never been
   '   opened -- used to set correct error info
   ' ----------------------------------------------
   mlngEventLogHwd = &HFFFFFFFF
End Sub




' -------------------------------------------------
' Set the direction to read the log -- forwards or
'   backwards.  Defaults to backwards, like the
'   NT Event Viewer
' -------------------------------------------------
Public Property Let EventReadLogForward(xlngDirectionToRead As Long)
   If xlngDirectionToRead = 0 Then
      mlngEventReadLogForward = False
   Else
      mlngEventReadLogForward = True
   End If
End Property

' -------------------------------------------------
' Private function to translate the embedded "%%"
'   parameters from the resource file with the
'   items returned from parameter file
' -------------------------------------------------
Private Function ResourceString2(xstrEventSourceName, xstrMsgArg As String) As String
   Dim pRegistryDB      As New RegistryDB
   Dim pstrLibName      As String
   Dim plngPos          As Long
   Dim plngPos2         As Long
   Dim pstrTmp          As String
   Dim plngItem         As Long
   Dim plngFlags        As Long
   Dim plngSize         As Long
   Dim pstrParameter    As String
   Dim plngModHwnd      As Long
   Dim plngRtn          As Long
   Dim pstrNewLibName   As String
   
   
   ' -------------------------------------------
   ' Turn off error trapping
   ' -------------------------------------------
   On Error Resume Next
   pstrTmp = xstrMsgArg
   
   
   ' -------------------------------------------
   ' Get info from the registry
   ' -------------------------------------------
   pRegistryDB.LogType = mstrTypeEventLog
   pRegistryDB.SourceName = xstrEventSourceName
   If pRegistryDB.RegistryEntry(ParameterMessage) = True Then
      pstrLibName = pRegistryDB.RegistryEntryText
   Else
      LastEventErrorNumber = pRegistryDB.LastRegError
      LastEventErrorSource = pRegistryDB.LastRegSource
      LastEventErrorDescription = pRegistryDB.LastRegDescription
      ResourceString2 = ""
      On Error GoTo 0
      Exit Function
   End If
   
   
   ' -------------------------------------------
   ' Replace replaceable parameter if there
   ' -------------------------------------------
   pstrLibName = UCase$(pstrLibName)
   plngPos = InStr(pstrLibName, "%")
   If plngPos > 0 Then
      pstrNewLibName = Space$(256)
      plngRtn = ExpandEnvironmentStrings(pstrLibName, pstrNewLibName, Len(pstrNewLibName))
      If plngRtn > 0 Then
         pstrLibName = Mid$(pstrNewLibName, 1, plngRtn)
      End If
   End If
   

   ' -------------------------------------------
   ' Replace each parameter starting with %%
   '   (and get rid of CR-LF pairs)
   ' -------------------------------------------
   plngPos = 0
   plngPos2 = 0
   plngPos = InStr(pstrTmp, "%%")
   plngPos2 = InStr(pstrTmp$, vbCrLf)
   
   Do While plngPos > 0
      
      ' -------------------------------------------
      ' Get the number (pointer) into the
      '   ParameterMessageFile
      ' -------------------------------------------
      plngItem = CLng(Mid$(pstrTmp, plngPos + 2, (plngPos2 - 1) - (plngPos + 2) + 1))
      
      
      ' -------------------------------------------
      ' Load the library
      ' -------------------------------------------
      plngModHwnd = LoadLibrary(pstrLibName)

      If plngModHwnd > 0 Then
         
         ' ----------------------------------------
         ' Set the flags for the API call
         ' ----------------------------------------
         plngFlags = FORMAT_MESSAGE_FROM_HMODULE Or _
                     FORMAT_MESSAGE_IGNORE_INSERTS
         plngSize = 512
         pstrParameter = Space$(plngSize)
   
   
         ' ----------------------------------------
         ' Get the message string from the
         '   file
         ' ----------------------------------------
         plngRtn = FormatMessage(plngFlags, ByVal plngModHwnd, _
               plngItem, &H0&, _
               pstrParameter, plngSize, _
               ByVal 0)
         
   
         ' ----------------------------------------
         ' Check for errors
         ' ----------------------------------------
         If plngRtn = 0 Then
            LastEventErrorNumber = (ERR_RESOURCE_DATA_NOT_FOUND + vbObjectError)
            LastEventErrorSource = EVENT_SOURCENAME & "ResourceString"
            LastEventErrorDescription = ReturnApiErrString(Err.LastDllError)
            pstrParameter = ""
         Else
            pstrParameter = Trim$(Left$(pstrParameter, plngRtn))
            If Right$(pstrParameter, 2) = vbCrLf Then
               pstrParameter = Left$(pstrParameter, Len(pstrParameter) - 2)
            End If
         End If
      End If
      
      If plngPos = 1 Then
         pstrTmp = pstrParameter & Chr$(0) & Mid$(pstrTmp, plngPos2 + 2)
      Else
         pstrTmp = Mid$(pstrTmp, 1, plngPos - 1) & pstrParameter & Chr$(0) & Mid$(pstrTmp, plngPos2 + 2)
      End If
      
      ' ----------------------------------------
      ' Look for more parameters
      ' ----------------------------------------
      plngPos = InStr(pstrTmp, "%%")
      plngPos2 = InStr(pstrTmp$, vbCrLf)
   Loop


   ' -------------------------------------------
   ' Add back the CR-LF pairs
   '   (and get rid of CR-LF pairs)
   ' -------------------------------------------
   plngPos = InStr(pstrTmp, Chr$(0))
   Do While plngPos > 0
      pstrTmp$ = Mid$(pstrTmp, 1, plngPos - 1) & vbCrLf & Mid$(pstrTmp, plngPos + 1)
      
      ' ----------------------------------------
      ' Look for more parameters
      ' ----------------------------------------
      plngPos = InStr(pstrTmp, Chr$(0))
   Loop
   

   ' -------------------------------------------
   ' Set the return value
   ' -------------------------------------------
   ResourceString2 = pstrTmp


   ' -------------------------------------------
   ' Reset error trapping
   ' -------------------------------------------
   On Error GoTo 0
   
End Function

' -------------------------------------------------
' Read the setting of whether or not to return HEX
'   values for data
' -------------------------------------------------
Public Property Get EventDataReturnHex() As Long
Attribute EventDataReturnHex.VB_HelpID = 23
Attribute EventDataReturnHex.VB_Description = "False = return hex for Data items less than 256 bytes and untranslated strings for items over 256 bytes"
   EventDataReturnHex = mlngEventDataReturnHex
End Property

' -------------------------------------------------
' Set whether or not to return HEX values for the
'   data -- will only effect data over 256 bytes in
'   length (ie, if set to FALSE, the default, then
'   data 256 bytes or less will *still* be returned
'   as hex values separated by spaces)
' -------------------------------------------------
Public Property Let EventDataReturnHex(xlngEventDataReturnHex As Long)
   If xlngEventDataReturnHex = False Then
      mlngEventDataReturnHex = False
   Else
      mlngEventDataReturnHex = True
   End If
End Property
