Attribute VB_Name = "Global_Class"
' *************************************************
' *        Copyright (C) L.J. Johnson, 1995       *
' *               All Rights Reserved.            *
' *************************************************

Option Explicit


' -------------------------------------------------
' Constants for different types of event filters
' -------------------------------------------------
Public Const Filter_Type_None = 0
Public Const Filter_Type_TimeBefore = 1
Public Const Filter_Type_TimeAfter = 2
Public Const Filter_Type_EventType = 3
Public Const Filter_Type_Source = 4
Public Const Filter_Type_Category = 5
Public Const Filter_Type_Computer = 6
Public Const Filter_Type_EventID = 7


' -------------------------------------------------
' The types of events that can be logged.
' -------------------------------------------------
Public Const EVENTLOG_SUCCESS = &H0
Public Const EVENTLOG_ERROR_TYPE = &H1
Public Const EVENTLOG_WARNING_TYPE = &H2
Public Const EVENTLOG_INFORMATION_TYPE = &H4
Public Const EVENTLOG_AUDIT_SUCCESS = &H8
Public Const EVENTLOG_AUDIT_FAILURE = &H10


' -------------------------------------------------
' Constants for different types of events
' -------------------------------------------------
Public Const Event_Type_Info = "Information"
Public Const Event_Type_Warning = "Warning"
Public Const Event_Type_Error = "Error"
Public Const Event_Type_Success_Audit = "Success Audit"
Public Const Event_Type_Failure_Audit = "Failure Audit"


' -------------------------------------------------
' Constants for the type of log you want to look
'   at.  Note that user program will need to use
'   these constants also
' -------------------------------------------------
Public Const EVNT_SYSTEM = "System"
Public Const EVNT_APP = "Application"
Public Const EVNT_SECURITY = "Security"


' -------------------------------------------------
' Custom error return codes for the various class
'   modules
' -------------------------------------------------
Public Const ERR_LOGTYPE_NOT_SET = 1011
Public Const ERR_SOURCENAME_NOT_SET = 1012
Public Const ERR_BAD_INDEX = 1013
Public Const ERR_FAILED_OPEN_REGISTRY_KEY = 1014
Public Const ERR_FAILED_READ_REGISTRY_KEY = 1015
Public Const ERR_RESOURCE_DATA_NOT_FOUND = 1016
Public Const ERR_READING_EVENT_LOG = 1017
Public Const ERR_LOG_NOT_OPENED = 1018
Public Const ERR_FAILED_SET_LOG_TYPE = 1019
Public Const ERR_FAILED_FORMAT_MSG = 1020


' -------------------------------------------------
' API calls that need to be public within program
' -------------------------------------------------
Public Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (dst As Any, src As Any, ByVal Size As Long)


' --------------------------------------------------------
' Eventlog Status Codes
' --------------------------------------------------------
Public Const ERROR_SUCCESS = 0&
Public Const ERROR_FILE_NOT_FOUND = 2&
Public Const ERROR_PATH_NOT_FOUND = 3&
Public Const ERROR_TOO_MANY_OPEN_FILES = 4&
Public Const ERROR_ACCESS_DENIED = 5&
Public Const ERROR_INVALID_HANDLE = 6&
Public Const ERROR_ARENA_TRASHED = 7&
Public Const ERROR_NOT_ENOUGH_MEMORY = 8&
Public Const ERROR_OUTOFMEMORY = 14&
Public Const ERROR_HANDLE_EOF = 38&
Public Const ERROR_INVALID_PARAMETER = 87&
Public Const ERROR_INSUFFICIENT_BUFFER = 122&
Public Const ERROR_INVALID_NAME = 123&
Public Const ERROR_ALREADY_EXISTS = 183&
Public Const ERROR_MORE_DATA = 234&
Public Const ERROR_NO_MORE_ITEMS = 259&
Public Const ERROR_MR_MID_NOT_FOUND = 317&
Public Const ERROR_NOACCESS = 998&
Public Const ERROR_BADDB = 1009&
Public Const ERROR_BADKEY = 1010&
Public Const ERROR_CANTOPEN = 1011&
Public Const ERROR_CANTREAD = 1012&
Public Const ERROR_CANTWRITE = 1013&
Public Const ERROR_EVENTLOG_FILE_CORRUPT = 1500&
Public Const ERROR_EVENTLOG_CANT_START = 1501&
Public Const ERROR_LOG_FILE_FULL = 1502&
Public Const ERROR_EVENTLOG_FILE_CHANGED = 1503&
Public Const RPC_S_SERVER_UNAVAILABLE = 1722&
Public Const ERROR_RESOURCE_DATA_NOT_FOUND = 1812&


' -------------------------------------------------
' Used to save the library names in static arrays
' -------------------------------------------------
'Type Event_Id_Type
'   EventID           As Long
'   LibName           As String
'   MsgString         As String
'End Type

Type Source_Lib_Type
   SourceName        As String
   LibName           As String
End Type

' -------------------------------------------------
' Pass:
'        Number of replacement strings
'        Original string to replace in (ie, %1, %2,
'              etc)
'        Array of replacement strings
' Returns:
'        Original string with replacements made
'              (xstrMsgString)
' -------------------------------------------------
Public Sub ReplaceParameters(xintNumStrings As Integer, xstrMsgString As String, xstrMsgArg() As String)
   Dim pstrTmpMsgString    As String
   Dim pintLoop            As Integer
   Dim plngPos             As Long
   Dim pstrFirst           As String
   Dim pstrLast            As String
   
   
   ' ----------------------------------------------
   ' Make a copy of the passed string
   ' ----------------------------------------------
   pstrTmpMsgString = xstrMsgString
   
   
   ' ----------------------------------------------
   ' Replace the replaceable parameters (ie, %1,
   '   %2, etc)
   ' ----------------------------------------------
   On Error Resume Next
   For pintLoop = 1 To xintNumStrings
      plngPos = InStr(pstrTmpMsgString, "%" & CStr(pintLoop))
      If plngPos > 0 Then
         pstrFirst = Left$(pstrTmpMsgString, plngPos - 1)
         If pintLoop < 10 Then
            pstrLast = Mid$(pstrTmpMsgString, plngPos + 2)
         ElseIf pintLoop < 100 Then
            pstrLast = Mid$(pstrTmpMsgString, plngPos + 3)
         Else
            pstrLast = Mid$(pstrTmpMsgString, plngPos + 4)
         End If
         pstrTmpMsgString = pstrFirst & xstrMsgArg(pintLoop - 1) & pstrLast
      End If
   Next pintLoop
   
   
   ' ----------------------------------------------
   ' Trim any trailing CR's, LF's or CrLf's
   ' ----------------------------------------------
   If Right$(pstrTmpMsgString, 2) = vbCrLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 2)
   ElseIf Right$(pstrTmpMsgString, 1) = vbLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   ElseIf Right$(pstrTmpMsgString, 1) = vbCr Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   End If
   
   
   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   xstrMsgString = pstrTmpMsgString

End Sub




' -------------------------------------------------
' Clear any trailing Cr's, Lf's or CrLf's from the
'   passed string.
' -------------------------------------------------
Public Function ClearLastCrLf(xstrTmpMsgString As String) As String
   Dim pstrTmpMsgString    As String
   
   
   ' ----------------------------------------------
   ' Make a temporary copy of the passed string
   ' ----------------------------------------------
   pstrTmpMsgString = Trim$(xstrTmpMsgString)
   
   
   ' ----------------------------------------------
   ' Trim any trailing nulls
   ' ----------------------------------------------
   If Right$(pstrTmpMsgString, 1) = Chr$(0) Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   End If
   
   
   ' ----------------------------------------------
   ' Trim any trailing Cr's, Lf's, or CrLf's
   ' ----------------------------------------------
   If Right$(pstrTmpMsgString, 2) = vbCrLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 2)
   ElseIf Right$(pstrTmpMsgString, 1) = vbLf Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   ElseIf Right$(pstrTmpMsgString, 1) = vbCr Then
      pstrTmpMsgString = Mid$(pstrTmpMsgString, 1, Len(pstrTmpMsgString) - 1)
   End If

   
   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ClearLastCrLf = pstrTmpMsgString
   
End Function




' -------------------------------------------------
' Note:  Passing strings to function, the trailing
'        Chr$(0)'s will be truncated -- need to add
'        them back
' Also, in one case, we will be passing only 2
'        characters to the function in order to get
'        the low-order number for EventID (so it
'        will be the same as shown in the Event
'        Viewer)
' -------------------------------------------------
Public Function CVL(xstrNumber As String) As Long
   Dim plngTmp    As Long
   
   Select Case Len(Trim(xstrNumber))
      Case 0
         Exit Function
      Case 1
         xstrNumber = xstrNumber & Chr$(0) & Chr$(0) & Chr$(0)
      Case 2
         xstrNumber = xstrNumber & Chr$(0) & Chr$(0)
      Case 3
         xstrNumber = xstrNumber & Chr$(0)
      Case Is > 4
         Exit Function
   End Select
   
   CopyMem plngTmp, ByVal xstrNumber, 4
   CVL = plngTmp
   
End Function


' -------------------------------------------------
' Note:  Passing strings to function, the trailing
'        Chr$(0)'s will be truncated -- need to add
'        them back
' -------------------------------------------------
Public Function CVI(xstrNumber As String) As Integer
   Dim pintTmp    As Integer
   
   Select Case Len(Trim(xstrNumber))
      Case 0
         Exit Function
      Case 1
         xstrNumber = xstrNumber & Chr$(0)
      Case Is > 2
         Exit Function
   End Select
   
   CopyMem pintTmp, ByVal xstrNumber, 2
   CVI = pintTmp
   
End Function



' -------------------------------------------------
' Needs a Sub Main, even if it doesn't do anything
' NOTE:  The commented out code can be used to test
'        the server locally, without calling it
'        from a client location
' -------------------------------------------------
Public Sub Main()
   
   ' ----------------------------------------------
   ' Set the Title to nothing so that it won't show
   '   up in the task list for some user to shut it
   '   down/.
   ' ----------------------------------------------
   App.Title = ""
   
   
   'Dim pEventLog As New EventLogs
   'Dim plngRtn As Variant
   '
   ''Debug.Print
   ''Debug.Print "******************************"
   ''Debug.Print "Open Event Log: " & Time
   'pEventLog.EventTypeLog = "System"
   'pEventLog.EventTypeLog = "Application"
   'pEventLog.EventTypeLog = "Security"
   'pEventLog.EventDataReturnHex = False
   ''If pEventLog.OpenAnyEventLog("\lj_nt_srv") = False Then
   'pEventLog.EventFilter(Filter_Type_EventID) = 4097
   'If pEventLog.OpenAnyEventLog("") = False Then
   '   Debug.Print pEventLog.LastEventErrorNumber, pEventLog.LastEventErrorSource
   '   Debug.Print pEventLog.LastEventErrorDescription
   'End If
   '
   '
   'pEventLog.EventReadLogForward = False
   'If pEventLog.ReadEventEntries = False Then
   '   Debug.Print "Error Number: " & pEventLog.LastEventErrorNumber
   '   Debug.Print "Error Source: " & pEventLog.LastEventErrorSource
   '   Debug.Print "Error Description: " & pEventLog.LastEventErrorDescription
   '   Exit Sub
   'End If
   'Debug.Print "End reading Event Log: " & Time
   '
   '
   'Debug.Print "Number of records: " & pEventLog.CountEventRecords
   'Dim i%
   'For i% = 1 To pEventLog.CountEventRecords
   '   Debug.Print i% & "  *********************************      "
   '   Debug.Print pEventLog.EventTimeWritten(CLng(i%))
   '   Debug.Print pEventLog.EventSourceName(CLng(i%))
   '   Debug.Print pEventLog.EventUserName(CLng(i%))
   '   Debug.Print pEventLog.EventComputerName(CLng(i%))
   '   Debug.Print pEventLog.EventType(CLng(i%))
   '   Debug.Print pEventLog.EventDescription(CLng(i%))
   '   Debug.Print pEventLog.EventData(CLng(i%))
   '   Debug.Print pEventLog.EventCategory(CLng(i%))
   '   Debug.Print pEventLog.EventRecordNum(CLng(i%))
   '   Debug.Print pEventLog.EventID(CLng(i%))
   'Next i%
  
End Sub




' -------------------------------------------------
' Passed an API error number, return an error
'   string
' Note the 2 replaceable parameters, which are used
'   currently only for the ERROR_MR_MID_NOT_FOUND
'   error which has 2 replaceable parameters
' -------------------------------------------------
Public Function ReturnApiErrString(ErrorCode As Long, Optional xvntMsgNum As Variant, Optional xvntMsgFile As Variant) As String
   Dim pstrMsgArray()   As String
   
   Select Case ErrorCode
      Case ERROR_SUCCESS
         ReturnApiErrString = "Success -- no error."
      Case ERROR_FILE_NOT_FOUND
         ReturnApiErrString = "The system cannot find the file specified."
      Case ERROR_PATH_NOT_FOUND
         ReturnApiErrString = "The system cannot find the path specified."
      Case ERROR_TOO_MANY_OPEN_FILES
         ReturnApiErrString = "Too many open files -- The system cannot open another file."
      Case ERROR_ACCESS_DENIED
         ReturnApiErrString = "Access is denied."
      Case ERROR_INVALID_HANDLE
         ReturnApiErrString = "The handle is invalid."
      Case ERROR_ARENA_TRASHED
         ReturnApiErrString = "The storage control blocks were destroyed."
      Case ERROR_NOT_ENOUGH_MEMORY
         ReturnApiErrString = "Not enough memory -- Not enough storage is available to process this command."
      Case ERROR_OUTOFMEMORY
         ReturnApiErrString = "Not enough storage is available to complete this operation."
      Case ERROR_HANDLE_EOF
         ReturnApiErrString = "Reached end of file."
      Case ERROR_INVALID_PARAMETER
         ReturnApiErrString = "The parameter is incorrect."
      Case ERROR_INSUFFICIENT_BUFFER
         ReturnApiErrString = "The data area passed to a system call is too small."
      Case ERROR_INVALID_NAME
         ReturnApiErrString = "The system call level is not correct."
      Case ERROR_ALREADY_EXISTS
         ReturnApiErrString = "Cannot create a file when that file already exists."
      Case ERROR_MORE_DATA
         ReturnApiErrString = "More data is available."
      Case ERROR_NO_MORE_ITEMS
         ReturnApiErrString = "No more data is available."
      Case ERROR_NOACCESS
         ReturnApiErrString = "Invalid access to memory location."
      Case ERROR_BADDB
         ReturnApiErrString = "The configuration registry database is corrupt."
      Case ERROR_BADKEY
         ReturnApiErrString = "The configuration registry key is invalid."
      Case ERROR_CANTOPEN
         ReturnApiErrString = "The configuration registry key could not be opened."
      Case ERROR_CANTREAD
         ReturnApiErrString = "The configuration registry key could not be read."
      Case ERROR_CANTWRITE
         ReturnApiErrString = "The configuration registry key could not be written."
      Case ERROR_EVENTLOG_FILE_CORRUPT
         ReturnApiErrString = "The event log file is corrupt."
      Case ERROR_EVENTLOG_CANT_START
         ReturnApiErrString = "No event log file could be opened, so the event logging service did not start."
      Case ERROR_LOG_FILE_FULL
         ReturnApiErrString = "The event log file is full."
      Case ERROR_EVENTLOG_FILE_CHANGED
         ReturnApiErrString = "The event log file has changed between reads."
      Case ERROR_RESOURCE_DATA_NOT_FOUND
         ReturnApiErrString = "The specified image file did not contain a resource section."
      Case RPC_S_SERVER_UNAVAILABLE
         ReturnApiErrString = "The specified server is not available."
      Case ERROR_MR_MID_NOT_FOUND
         ReturnApiErrString = "The system cannot find message for message " & _
                              "number %1 in message file for %2."
         ReDim pstrMsgArray(0 To 1) As String
         pstrMsgArray(1) = CStr(xvntMsgNum)
         pstrMsgArray(2) = CStr(xvntMsgFile)
         ReplaceParameters 2, ReturnApiErrString, pstrMsgArray()
      Case Else
         ReturnApiErrString = "Unknown error #" & CStr(ErrorCode)
   End Select

End Function
