Attribute VB_Name = "Codex_Module_01"
Option Explicit

#If win32 Then
    Declare Function GetTickCount Lib "kernel32" () As Long
#Else
    Declare Function GetTickCount Lib "User" () As Long
#End If
Public Const MON_DEBUG = 1
Public Const MON_MSGBOX = 2
Public Const MON_PRINTER = 3
#If win32 Then
    Declare Sub GlobalMemoryStatus Lib "kernel32" (lpBuffer As MEMORYSTATUS)
    Type MEMORYSTATUS
        dwLength As Long            ' sizeof(MEMORYSTATUS)
        dwMemoryLoad As Long        ' percent of memory in use
        dwTotalPhys As Long         ' bytes of physical memory
        dwAvailPhys As Long         ' free physical memory bytes
        dwTotalPageFile As Long     ' bytes of paging file
        dwAvailPageFile As Long     ' free bytes of paging file
        dwTotalVirtual As Long      ' user bytes of address space
        dwAvailVirtual As Long      ' free user bytes
    End Type
    Dim MemStat As MEMORYSTATUS
#Else
    Declare Function GetFreeSystemResources Lib "User" (ByVal fuSysResource As Integer) As Integer
    Declare Function GetFreeSpace Lib "Kernel" (ByVal wFlags As Integer) As Long
    Const GDI_RESOURCES As Integer = 1
    Const USER_RESOURCES As Integer = 2
#End If

'Printer Fonts
Global gPrnTitRepFontName As String
Global gPrnTitRepFontSize As Integer
Global gPrnTxtRepFontName As String
Global gPrnTxtRepFontSize As Integer
Global gPrnArray(15)
Public Const TIME_DEBUG = 1
Public Const TIME_MSGBOX = 2
Public Const TIME_PRINTER = 3
Global glTimeBegin As Long
Global glTimeEnd As Long
Global gsTimeCaption As String
Global giTimePrintMode As Integer
Type ProfileStorage
    TotalTime As Long
    Iterations As Long
    Caption As String
End Type
Dim StoreTime() As ProfileStorage

'Begins a timing session
Public Sub BeginTime(Caption As String, mode As Integer)

gsTimeCaption = Caption
giTimePrintMode = mode
glTimeBegin = GetTickCount()

End Sub

'Draws a line to printer - belongs to QuickReport
Sub DrawLine()
   
    Printer.DrawWidth = 2
    Printer.Line -(Printer.ScaleWidth, Printer.CurrentY)
    Printer.CurrentX = 0

End Sub


'Draws a double line - belongs to QuickReport
Public Sub DrawLine2()
    
    Printer.DrawWidth = 2
    Printer.Line -(Printer.ScaleWidth, Printer.CurrentY)
    Printer.CurrentX = 0
    Printer.CurrentY = Printer.CurrentY + (Printer.TwipsPerPixelY * 5)
    Printer.Line -(Printer.ScaleWidth, Printer.CurrentY)
    Printer.CurrentX = 0

End Sub


'Completes profile timing and shows results
Public Sub EndTime()
Static Init As Integer
Dim foo As Integer, foobar As Integer

glTimeEnd = GetTickCount()

Select Case giTimePrintMode
Case 0 'do nothing - disabled
Case 1 'Debug.print Detail
    Debug.Print Format$((glTimeEnd - glTimeBegin) / 1000#, "###0.0000"), gsTimeCaption
Case 2 'Message Box
    MsgBox gsTimeCaption & Chr$(10) & Format$((glTimeEnd - glTimeBegin) / 1000#, "###0.0000"), , "Time Profile"
Case 3 'Printer.Print Detail
    If Init = False Then
        Init = True
        gPrnArray(0) = "Profile: " & App.EXEName
        gPrnArray(1) = "Seconds"
        gPrnArray(2) = "Profile Description"
        QuickReport "N", 6, gPrnArray(), "P"
    End If
    Erase gPrnArray()
    gPrnArray(0) = Format$((glTimeEnd - glTimeBegin) / 1000#, "###0.0000")
    gPrnArray(1) = gsTimeCaption
    QuickReport "D", 6, gPrnArray(), "P"
Case 4 'Printer Stop
    Printer.EndDoc
End Select

End Sub

'Measures resource and RAM usage
Public Sub Monitor(Caption As String, DisplayMode As Integer)
'DisplayMode    0 = disabled
'               1 = Debug window
'               2 = MsgBox
'               3 = Printer
Dim foo As Integer, foobar As Integer
Dim FreeMem As Long
Dim FreeGDI As Integer
Dim FreeUser As Integer
Dim DisplayStr As String
If DisplayMode = 0 Then Exit Sub
#If win32 Then
    MemStat.dwLength = 32
    Call GlobalMemoryStatus(MemStat)
    FreeMem = MemStat.dwAvailPhys
    FreeGDI = 50 '<<<<<< Need info on updated API calls for Win95
    FreeUser = 50
    DisplayStr = "GDI: " & FreeGDI & "%   User: " & FreeUser & "%   Mem: " & (FreeMem \ 1024) & "K"
#Else
    FreeMem = GetFreeSpace(0)
    FreeGDI = GetFreeSystemResources(GDI_RESOURCES)
    FreeUser = GetFreeSystemResources(USER_RESOURCES)
    DisplayStr = "GDI: " & FreeGDI & "%   User: " & FreeUser & "%   Mem: " & (FreeMem \ 1024) & "K"
#End If
Select Case DisplayMode
Case 1 'Debug.print
    Debug.Print "Free Resources: " & Caption
    Debug.Print DisplayStr
Case 2 'msgbox
    MsgBox Caption & Chr$(10) & Chr$(13) & DisplayStr, 0, "Free Resources"
Case 3 'printer
    Printer.Print "Free Resources: " & Caption;
    Printer.CurrentX = Printer.ScaleWidth / 2
    Printer.Print DisplayStr
End Select
End Sub

'Prints formatted reports, quick and easy
Public Sub QuickReport(mode As String, printzones, printarray(), printmode As String)
Static page As Integer
Dim foobar As Integer, foo As Integer

'Init Fonts if not set
If Trim$(gPrnTitRepFontName) = "" Then gPrnTitRepFontName = "Arial"
If gPrnTitRepFontSize = 0 Then gPrnTitRepFontSize = 12
If Trim$(gPrnTxtRepFontName) = "" Then gPrnTxtRepFontName = "Arial"
If gPrnTxtRepFontSize = 0 Then gPrnTxtRepFontSize = 8

Static lines As Integer
Static HeaderLines(15) As String
Static HeaderLines2(15) As String
Static H2Flag As Integer
Static NewRep As Integer
Static HeaderTitle As String
Dim printstring As String
Dim DateString As String

If printmode = "P" Then
    'New Report - Define Title and Column Headers
    If mode = "N" Then
        NewRep = True
        H2Flag = False
        page = 0
        lines = 0
        HeaderTitle = printarray(0)
        For foobar = 0 To 15
            HeaderLines(foobar) = ""
        Next foobar
        For foobar = 0 To printzones
            HeaderLines(foobar) = printarray(foobar + 1)
        Next foobar
    End If
    'Second Header Define
    If mode = "H2" Then
        H2Flag = True
        For foobar = 0 To 15
            HeaderLines2(foobar) = ""
        Next foobar
        For foobar = 0 To printzones
            HeaderLines2(foobar) = printarray(foobar + 1)
        Next foobar
    End If
    'Time For a Header Line?
    If mode = "H" Or (mode = "D" And NewRep = True) Or Printer.CurrentY >= (Printer.Height - 2160) Then
        page = page + 1
        NewRep = False
        lines = 5
        If page <> 1 Then
            Printer.NewPage
        End If
        'Print the title line
        Printer.FontName = gPrnTitRepFontName
        Printer.FontSize = gPrnTxtRepFontSize
        Printer.Print ""
        Printer.FontSize = gPrnTitRepFontSize
        Printer.CurrentX = ((Printer.ScaleWidth) / 2) - (Printer.TextWidth(HeaderTitle) / 2)
        Printer.Print HeaderTitle;
        ' Print page and date even with title line
        Printer.FontName = gPrnTxtRepFontName
        Printer.CurrentY = Printer.CurrentY + (Printer.TextHeight("X") / 4)
        Printer.FontSize = gPrnTxtRepFontSize
        DateString = Format$(Now, "MM/DD/YY HH:MM AM/PM")
        Printer.CurrentX = (Printer.ScaleWidth) - Printer.TextWidth(DateString)
        Printer.Print DateString;
        Printer.CurrentX = 0
        printstring = Trim$(" Page " & Trim$(Format$(page, "###0")))
        Printer.Print printstring;
        Printer.CurrentY = Printer.CurrentY + Printer.TextHeight("X") * 1.3
        Printer.CurrentX = 0
        DrawLine
        Printer.CurrentY = Printer.CurrentY + (Printer.TextHeight("X") / 2)
        For foobar = 0 To printzones - 1
            Printer.CurrentX = foobar * ((Printer.ScaleWidth) / (printzones))
            Printer.Print HeaderLines(foobar);
        Next foobar
        Printer.Print ""
        If H2Flag = True Then
            For foobar = 0 To printzones - 1
                Printer.CurrentX = foobar * ((Printer.ScaleWidth) / (printzones))
                Printer.Print HeaderLines2(foobar);
            Next foobar
            Printer.Print ""
            lines = lines + 1
        End If
        Printer.CurrentY = Printer.CurrentY + (Printer.TextHeight("X") / 2)
        DrawLine
        Printer.Print ""
    End If
    'Detail Line
    If mode = "D" Then
        For foobar = 0 To printzones - 1
            Printer.CurrentX = foobar * ((Printer.ScaleWidth) / (printzones))
            Printer.Print printarray(foobar);
        Next foobar
        Printer.Print ""  'Next line
        lines = lines + 1
    End If
    'Blank Line
    If mode = "B" Then
        Printer.Print ""
        lines = lines + 1
    End If
    'Single Line
    If mode = "1" Then
        DrawLine
        Printer.Print ""
        lines = lines + 1
    End If
    'Double Line
    If mode = "2" Then
        DrawLine2
        Printer.Print ""
        lines = lines + 1
    End If
Else
    'Write ascii file if detail line
    If mode = "D" Then
        printstring = ""
        For foo = 0 To printzones - 2
            printstring = printstring & printarray(foo) & ","
        Next foo
        printstring = printstring & printarray(printzones - 1)
        Print #Val(printmode), printstring
        Debug.Print printstring
    End If
End If
End Sub

