Attribute VB_Name = "RDOTest"
Option Explicit

'  Written by Steve Jackson
'  Huntington Beach, California
'  Compuserve: 72040,1640
'  This program was used for the database testing in Steve's
'  article in Visual Basic Programmers Journal, May 1996


#If Win32 Then
' bracket the entire module with
' #If Win32 and #EndIf so that
' it only compiles in 32-bit mode.
' Otherwise it will cause compile
' errors in VB4 16-bit mode

Dim rdoEnv As rdoEnvironment
Dim rdoConn As rdoConnection
Dim sSQLStmt As String
Dim rdoRes As rdoResultset

Dim rdoPStmtSelectByKey As rdoPreparedStatement
Dim rdoPStmtInsert As rdoPreparedStatement
Dim rdoPStmtDelete As rdoPreparedStatement

Dim mbPreparedStatement As Integer

Sub DoRDOTest(bPreparedStatement As Integer)
    Dim nRows As Long
    Dim sData As String
    Dim i As Integer
    Dim lStart As Long, lEnd As Long
    Dim iMax As Integer
  
    Screen.MousePointer = 11
    Call ClearForm
    iMax = 20
    
    ShowStatus "Starting Test"
    ShowTotMsg "Starting Test"
    If bPreparedStatement = 1 Then
        mbPreparedStatement = True
        ShowStatus "32-bit RDO using prepared statements"
        ShowTotMsg "32-bit RDO using prepared statements"
    Else
        mbPreparedStatement = False
        ShowStatus "32-bit RDO (NOT using prepared statements)"
        ShowTotMsg "32-bit RDO (NOT using prepared statements)"
    End If
    
    StartTimeCount "Start connection"
    lStart = GetTickCount()

    Call RdoStartConnection
    ShowStatus (EndTimeCount())
    
    '  Create a prepared statement for
    '  select customer by key value
    '  1 parameter = CustID
    If mbPreparedStatement Then
     sSQLStmt = "SELECT LastName, FirstName, LastOrderDate, TimesOrderPlaced from customer where CustID = ?"
     Set rdoPStmtSelectByKey = rdoConn.CreatePreparedStatement("SelectCust", sSQLStmt)
     ' insert into customer values("A1", "Jack", "x", "x", "x", 21)
     sSQLStmt = "insert into customer values(?, ?, ?, ?, ?, ?)"
     Set rdoPStmtInsert = rdoConn.CreatePreparedStatement("InsertCust", sSQLStmt)
     sSQLStmt = "delete customer where CustID = ?"
     Set rdoPStmtDelete = rdoConn.CreatePreparedStatement("DeleteCust", sSQLStmt)
    End If

    '   SELECT tests --------------------
    StartTimeCount "Select row by indexed key"
    sData = RdoGetRowByKey("19099")
    ShowStatus (EndTimeCount())
    ShowStatus "        Row: " & sData
       
    StartTimeCount "Select different row by indexed key"
    sData = RdoGetRowByKey("19086")
    ShowStatus (EndTimeCount())
    ShowStatus "        Row: " & sData

    StartTimeCount "Select first row again by indexed key"
    sData = RdoGetRowByKey("19099")
    ShowStatus (EndTimeCount())
    ShowStatus "        Row: " & sData
       
    Call CalcRandKeys
    StartTimeCount "SELECT " & Str$(iMax) & " ROWS by indexed key"
    For i = 1 To iMax
        sData = RdoGetRowByKey(msRandKey(i))
    Next i
    ShowStatus (EndTimeCount())
    
    StartTimeCount "SELECT ALL ROWS (200 row table)"
    nRows = RdoReadAllRows()
    ShowStatus (EndTimeCount())
    ShowStatus "        Rows: " & Str$(nRows)
    
    StartTimeCount "SELECT ROWS by non-indexed search value"
    nRows = RdoGetRowBySearchValue("117")
    ShowStatus (EndTimeCount())
    ShowStatus "        Rows: " & Str$(nRows)

      '   UPDATE tests -----------------------------------
    msDateToday = Date$
    StartTimeCount "Update 1 row by indexed key value"
    nRows = RdoUpdateRow("22")
    ShowStatus (EndTimeCount())
    If nRows = 0 Then ShowStatus "        ERROR updating row"
    
    Call CalcRandKeys
    StartTimeCount "UPDATE " & Str$(iMax) & " ROWS by indexed key value"
    For i = 1 To iMax
        nRows = RdoUpdateRow(msRandKey(i))
        If nRows = 0 Then ShowStatus "        ERROR updating row"
    Next i
    ShowStatus (EndTimeCount())
    
    Call CalcNewKeys
    ' if needed - delete any records left over
    ' from the last round of insert tests
    'nRows = RdoDeleteRow("1A")
    'For i = 1 To iMax
    '    nRows = RdoDeleteRowPassthru(msRandKey(i))
    'Next i

    i = 0
    '   OLD INSERT STUFF
    '   For using .AddNew and .Edit and .Update into the resultset
    'StartTimeCount "Open result set for INSERT"
    'Set rdoRes = rdoConn.OpenResultset("Select * from Customer", rdOpenForwardOnly, rdConcurRowver)
    'ShowStatus (EndTimeCount())
    ' insert calls where here that assumed an insert recordset is open
    'rdoRes.Close    ' close the insert result set
    'ShowStatus (EndTimeCount())
    
    '  INSERT Using SQL Pass Through ----------------
    StartTimeCount "Insert 1 row"
    nRows = RdoInsertRow("1A")
    ShowStatus (EndTimeCount())
    If nRows = 0 Then ShowStatus "        ERROR inserting row"
    
    StartTimeCount "INSERT " & Str$(iMax) & " ROWS"
    For i = 1 To iMax
        nRows = RdoInsertRow(msRandKey(i))
        If nRows = 0 Then ShowStatus "        ERROR inserting row"
    Next i
    ShowStatus (EndTimeCount())
    
    '    DELETE  -------------------------------------
    StartTimeCount "Delete 1 row"
    nRows = RdoDeleteRow("1A")
    ShowStatus (EndTimeCount())
    If nRows = 0 Then ShowStatus "        ERROR deleting row"
    
    StartTimeCount "DELETE " & Str$(iMax) & " ROWS"
    For i = 1 To iMax
        nRows = RdoDeleteRow(msRandKey(i))
        If nRows = 0 Then ShowStatus "        ERROR deleting row"
    Next i
    ShowStatus (EndTimeCount())
    
    '    JOIN   ------------------------------------------
    StartTimeCount "SELECT 2 TABLE JOIN"
    nRows = RdoSelectJoin("98")
    ShowStatus (EndTimeCount())
    ShowStatus "        Rows: " & Str$(nRows)

    StartTimeCount "Closing database connection"
    Call RdoEndConnection
    ShowStatus (EndTimeCount())

    lEnd = GetTickCount()

    Screen.MousePointer = 0
    ShowStatus "Done, total time = " & Str$((lEnd - lStart) / 1000)
    ShowStatus "------------------------------------"
    ShowTotMsg "Done, total time = " & Str$((lEnd - lStart) / 1000)
End Sub




Function RdoDeleteRow(sKeyValue As String) As Long
    Dim sKey As String * 10

    sKey = sKeyValue
    On Local Error GoTo RdoDeleteRowError
    If mbPreparedStatement Then
        rdoPStmtDelete.rdoParameters(0) = LTrim$(sKey)
        rdoPStmtDelete.Execute
    Else
        sSQLStmt = "Delete Customer where CustID = " & SQ & sKey & SQ
        rdoConn.Execute sSQLStmt
    End If
    
    ' yet another way to do it, but very slow....
    'sSQLStmt = "Select * from Customer where CustID = " & SQ & sKey & SQ
    'Set rdoRes = rdoConn.OpenResultset(sSQLStmt, rdOpenKeyset, rdConcurRowver)

    'If rdoRes.EOF Then
    '    rdoRes.Close
    '    RdoDeleteRow = 0
    '    Exit Function
    'End If
    
    'rdoRes.Edit
    'rdoRes.Delete
    'rdoRes.Close
    RdoDeleteRow = 1
    Exit Function
    
RdoDeleteRowError:
    RdoDeleteRow = 0
    Exit Function

End Function

Sub RdoEndConnection()
    On Error GoTo RdoEndConnectionErr
    rdoConn.Close
    
    'RdoEndConnection = 0
    Exit Sub

RdoEndConnectionErr:
    MsgBox Error$(Err), 48
    'RdoEndConnection = 1
    Exit Sub
End Sub

Function RdoGetRowByKey(sKeyValue As String) As String
    Dim sKey As String * 10

    sKey = sKeyValue
   
    If mbPreparedStatement Then
        rdoPStmtSelectByKey.rdoParameters(0) = sKey
        Set rdoRes = rdoPStmtSelectByKey.OpenResultset()
    Else
        sSQLStmt = "Select LastName, FirstName from Customer where CustID = " & SQ & sKey & SQ
        Set rdoRes = rdoConn.OpenResultset(sSQLStmt)
    End If

    If Not rdoRes.EOF Then
        RdoGetRowByKey = RTrim$(rdoRes("LastName")) & ", " & RTrim$(rdoRes("FirstName"))
    Else
        RdoGetRowByKey = "ERROR - NONE FOUND!"
    End If
        
    rdoRes.Close
End Function

Function RdoGetRowBySearchValue(sSearchValue As String) As Long
    Dim nRows As Long
    Dim sData As String
    Dim sValue As String * 10

    sValue = sSearchValue
    sSQLStmt = "Select ProductName from Product where ProductCode = " & SQ & sValue & SQ
    Set rdoRes = rdoConn.OpenResultset(sSQLStmt)
    
    nRows = 0
    While Not rdoRes.EOF
        sData = rdoRes("ProductName")
        rdoRes.MoveNext
        nRows = nRows + 1
    Wend
    rdoRes.Close

    RdoGetRowBySearchValue = nRows

End Function

Function RdoInsertRow(sKeyValue As String) As Long
    Dim sKey As String * 10
    
    sKey = sKeyValue
    On Local Error GoTo RdoInsertRowError
    
    If mbPreparedStatement Then
        rdoPStmtInsert.rdoParameters(0) = sKey
        rdoPStmtInsert.rdoParameters(1) = "RDO"
        rdoPStmtInsert.rdoParameters(2) = "PStmt"
        rdoPStmtInsert.rdoParameters(3) = "x"
        rdoPStmtInsert.rdoParameters(4) = "c"
        rdoPStmtInsert.rdoParameters(5) = "1"
        rdoPStmtInsert.Execute
    Else
        ' insert into customer values("A1", "Jack", "x", "x", "x", 21)
        sSQLStmt = "INSERT Into Customer values("
        sSQLStmt = sSQLStmt & SQ & sKey & SQ & ", "
        sSQLStmt = sSQLStmt & SQ & "test" & SQ & ", "
        sSQLStmt = sSQLStmt & SQ & "test" & SQ & ", "
        sSQLStmt = sSQLStmt & SQ & "test" & SQ & ", "
        sSQLStmt = sSQLStmt & SQ & "test" & SQ & ", "
        sSQLStmt = sSQLStmt & " 1)"
        rdoConn.Execute (sSQLStmt)
    End If
    
    RdoInsertRow = 1 ' rdoConn.RowsAffected
    Exit Function
    
    ' yet another way to do it, but slower...
    ' (assuming a resultset is open)
    'rdoRes.AddNew
    'rdoRes("CustID") = sKey
    'rdoRes("LastName") = "New"
    'rdoRes("FirstName") = "Test stuff"
    'rdoRes("State") = "CA"
    'rdoRes("LastOrderDate") = msDateToday
    'rdoRes("TimesOrderPlaced") = 1
    
    'rdoRes.Update
    'rdoRes.Close
    
RdoInsertRowError:
    RdoInsertRow = 0
    Exit Function
End Function





Function RdoSelectJoin(sKeyValue As String) As Long
    Dim nRows As Long
    Dim sData As String
    Dim sValue As String * 10

    ' since we do this only once,
    ' no prepared statement
    sValue = sKeyValue
    sSQLStmt = "select LastName, OrderDate, Shipdate from customer, orderentry  where customer.CustID = " & SQ & sValue & SQ & " and customer.custid = orderentry.custid"
    Set rdoRes = rdoConn.OpenResultset(sSQLStmt)
    
    nRows = 0
    While Not rdoRes.EOF
        sData = rdoRes("LastName")
        sData = rdoRes("OrderDate")
        sData = rdoRes("ShipDate")
        rdoRes.MoveNext
        nRows = nRows + 1
    Wend
    rdoRes.Close

    RdoSelectJoin = nRows

End Function

Sub RdoStartConnection()
    Dim sConn As String
    On Error GoTo RdoStartConnectionErr
    
    sConn = MY_ODBC_API_CONNECT
    Set rdoEnv = rdoEnvironments(0)
    Set rdoConn = rdoEnv.OpenConnection("", rdDriverComplete, False, sConn)
    
    'RdoStartConnection = 0
    Exit Sub

RdoStartConnectionErr:
    MsgBox Error$(Err), 48
    'RdoStartConnection = 1
    Exit Sub
End Sub

Function RdoReadAllRows() As Long
    Dim sData As String
    Dim nRows As Long
 
  
    sSQLStmt = "SELECT ProductCode, ProductDescription FROM ProductCode"
    Set rdoRes = rdoConn.OpenResultset(sSQLStmt)
    
    nRows = 0
    While Not rdoRes.EOF
        sData = rdoRes("ProductCode")
        rdoRes.MoveNext
        If nRows < 20 Then
            Form1!List2.AddItem sData
        End If
        nRows = nRows + 1
    Wend
    rdoRes.Close
    
    RdoReadAllRows = nRows
End Function



Function RdoUpdateRow(sKeyValue As String) As Long
    Dim sKey As String * 10
    Dim iTimesOrderPlaced As Integer

    sKey = sKeyValue
    If mbPreparedStatement Then
        rdoPStmtSelectByKey.rdoParameters(0) = sKey
        Set rdoRes = rdoPStmtSelectByKey.OpenResultset(rdOpenKeyset, rdConcurRowver)
    Else
        sSQLStmt = "Select LastOrderDate, TimesOrderPlaced from Customer where CustID = " & SQ & sKey & SQ
        Set rdoRes = rdoConn.OpenResultset(sSQLStmt, rdOpenKeyset, rdConcurRowver)
    End If

    If rdoRes.EOF Then
        rdoRes.Close
        RdoUpdateRow = 0
        Exit Function
    End If
    
    If IsNull(rdoRes("TimesOrderPlaced")) Then
        iTimesOrderPlaced = 0
    Else
        iTimesOrderPlaced = rdoRes("TimesOrderPlaced")
    End If
    iTimesOrderPlaced = iTimesOrderPlaced + 1
    
    rdoRes.Edit
    rdoRes("LastOrderDate") = msDateToday
    rdoRes("TimesOrderPlaced") = iTimesOrderPlaced
        
    rdoRes.Update
    rdoRes.Close
    RdoUpdateRow = 1
End Function


#End If




