VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "RegistryDB"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' *************************************************
' *        Copyright (C) L.J. Johnson, 1995       *
' *               All Rights Reserved.            *
' *************************************************

' -------------------------------------------------
' NOTE:  This class is private, used only
'        internally by this OLE object (not exposed
'        to the client)
' -------------------------------------------------

Option Explicit


' -------------------------------------------------
' Used as part of error string return
' -------------------------------------------------
Private Const REG_SOURCENAME = "ReadEventLogs.RegistryDB."


' -------------------------------------------------
' Type of item to look up
' -------------------------------------------------
Private Const EventMessage = 0
Private Const ParameterMessage = 1


' -------------------------------------------------
' Public -- These are used in the public class,
'           EventLogs
' -------------------------------------------------
Public LastRegError        As Long
Public LastRegSource       As String
Public LastRegDescription  As String


' -------------------------------------------------
' Module-level variables used to feed the Property
'   Get functions
' -------------------------------------------------
Private mstrLogType           As String
Private mstrSourceName        As String
Private mstrRegistryEntry     As String


' -------------------------------------------------
' These 32-bit API calls are used only in this
'   class module
' -------------------------------------------------
Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32" Alias _
   "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, _
   ByVal ulOptions As Long, ByVal samDesired As Long, _
   phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32" Alias _
   "RegQueryValueExA" (ByVal hKey As Long, _
   ByVal lpValueName As String, lpReserved As Long, _
   lptype As Long, lpData As Any, lpcbData As Long) As Long


' -------------------------------------------------
' These constants for main registry keys are used
'   only in this class module
' -------------------------------------------------
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006


' -------------------------------------------------
' Read/Write permissions for registry -- used only
'   in this class module
' -------------------------------------------------
Private Const KEY_QUERY_VALUE = &H1&
Private Const KEY_SET_VALUE = &H2&
Private Const KEY_CREATE_SUB_KEY = &H4&
Private Const KEY_ENUMERATE_SUB_KEYS = &H8&
Private Const KEY_NOTIFY = &H10&
Private Const KEY_CREATE_LINK = &H20&
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_READ = READ_CONTROL
Private Const STANDARD_RIGHTS_WRITE = READ_CONTROL
Private Const STANDARD_RIGHTS_EXECUTE = READ_CONTROL
Private Const KEY_READ = STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY
Private Const KEY_WRITE = STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY
Private Const KEY_EXECUTE = KEY_READ



' -------------------------------------------------
' Get the registry for EventMessageFile or
'   PamameterMessageLog entry --
'   have to set the Log Type and SourceName before
'   calling this Property
' Returns TRUE if successful, FALSE otherwise
' -------------------------------------------------
Public Function RegistryEntry(xlngType As Long) As Long
   Dim plngKeyType      As Long
   Dim plngRtn          As Long
   Dim plngOpenedKey    As Long
   Dim plngLenRtnData   As Long
   Dim pstrRtnString    As String
   Dim pstrValue        As String
   Dim pstrErrorStr     As String
   Dim pstrKey          As String
   Dim pstrValueName    As String
   
   
   ' -----------------------------------------------------
   ' Set the value for the Value Name
   ' -----------------------------------------------------
   If xlngType = ParameterMessage Then
      pstrValueName = "ParameterMessageFile"
   Else
      pstrValueName = "EventMessageFile"
   End If
   
   
   ' -----------------------------------------------------
   ' Check to make sure that the 2 variables have been set
   ' -----------------------------------------------------
   If CStr(mstrLogType) = CStr(&HFFFFFFFF) Then
      ' --------------------------------------------------
      ' Error info has already been set -- so just exit
      '   property with a FALSE return
      ' --------------------------------------------------
      RegistryEntry = False
      Exit Function
   ElseIf Len(Trim$(mstrLogType)) = 0 Then
      LastRegError = (ERR_LOGTYPE_NOT_SET + vbObjectError)
      LastRegSource = REG_SOURCENAME & "RegistryEntry"
      LastRegDescription = "The Log Type has not been set."
      RegistryEntry = False
      Exit Function
   ElseIf Len(Trim$(mstrSourceName)) = 0 Then
      LastRegError = (ERR_SOURCENAME_NOT_SET + vbObjectError)
      LastRegSource = REG_SOURCENAME & "RegistryEntry"
      LastRegDescription = "The Source Name has not been set."
      RegistryEntry = False
      Exit Function
   End If
 
 
   ' -----------------------------------------------------
   ' Turn off errors
   ' -----------------------------------------------------
   On Error Resume Next


   ' --------------------------------------------------
   ' Open key
   ' --------------------------------------------------
   pstrKey = "SYSTEM\CurrentControlSet\Services\EventLog\" _
              & mstrLogType & "\" & mstrSourceName
   plngRtn = RegOpenKeyEx(HKEY_LOCAL_MACHINE, pstrKey, 0&, KEY_READ, _
         plngOpenedKey)
   If plngRtn <> ERROR_SUCCESS Then
      LastRegError = (ERR_FAILED_OPEN_REGISTRY_KEY + vbObjectError)
      LastRegSource = REG_SOURCENAME & "RegistryEntry"
      LastRegDescription = "Error opening Registry Key, " & "'" & pstrKey & "'" _
                        & ".  Error was: " & ReturnApiErrString(Err.LastDllError)
      RegistryEntry = False
      Exit Function
   End If


   ' --------------------------------------------------
   ' Set up buffer for data to be returned in.
   ' --------------------------------------------------
   plngLenRtnData = 255
   pstrRtnString = Space$(plngLenRtnData)


   ' --------------------------------------------------
   ' Read key -- try again if buffer isn't big enough
   ' --------------------------------------------------
ValueTryAgain:
   plngRtn = RegQueryValueEx(plngOpenedKey, pstrValueName, ByVal 0&, _
         plngKeyType, ByVal pstrRtnString, plngLenRtnData)

   If plngRtn = ERROR_MORE_DATA Then
      pstrRtnString = Space$(plngLenRtnData)
      GoTo ValueTryAgain
   ElseIf plngRtn <> ERROR_SUCCESS Then
      LastRegError = (ERR_FAILED_READ_REGISTRY_KEY + vbObjectError)
      LastRegSource = REG_SOURCENAME & "RegistryEntry"
      LastRegDescription = "Error reading the Registry Entry, " _
                           & "'" & pstrKey & "'" & " for the Value Name, " _
                           & "'" & pstrValueName & "'.  Error was: " _
                           & ReturnApiErrString(Err.LastDllError)
      RegistryEntry = False
      Exit Function
   Else
      RegistryEntry = True
      mstrRegistryEntry = Left$(pstrRtnString, plngLenRtnData - 1)
   End If


   ' --------------------------------------------------
   ' Always close opened keys!
   ' --------------------------------------------------
   plngRtn = RegCloseKey(plngOpenedKey)

   On Error GoTo 0

End Function




' -------------------------------------------------
' Set the log type
' -------------------------------------------------
Public Property Let LogType(xstrLogType As String)
   Select Case xstrLogType
      Case EVNT_SYSTEM, EVNT_APP, EVNT_SECURITY
         mstrLogType = xstrLogType
      Case Else
         LastRegError = (ERR_FAILED_SET_LOG_TYPE + vbObjectError)
         LastRegSource = REG_SOURCENAME & "LogType"
         LastRegDescription = "Error setting the LogType: Passed: " & _
                              xstrLogType & ".  LogType must be 'Application'," _
                              & " 'Security', or 'System'."
         xstrLogType = CStr(&HFFFFFFFF)
   End Select
   
End Property


' -------------------------------------------------
' Retrieve the SourceName
' -------------------------------------------------
Public Property Let SourceName(xstrSourceName As String)
   mstrSourceName = xstrSourceName
End Property

' -------------------------------------------------
' Set 2 strings to zero-lengths on initialization
' -------------------------------------------------
Private Sub Class_Initialize()
   mstrLogType = ""
   mstrSourceName = ""
End Sub



' -------------------------------------------------
' Get the Value string for the registry value entry
' -------------------------------------------------
Public Property Get RegistryEntryText() As String
   RegistryEntryText = mstrRegistryEntry
End Property


