Attribute VB_Name = "Global_Module"
Option Explicit

Dim mvntFilterType      As Variant
Dim mvntFilterText      As Variant


' -------------------------------------------------
' API calls to stop constant re-drawing of the
'   List while filling
' -------------------------------------------------
Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Long) As Long
Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Public Const WM_SETREDRAW = &HB&


' -------------------------------------------------
' This Type corresponds to the one in the OLE
'   Server
' -------------------------------------------------
Type EventRecord
   EventTimeWritten  As Variant
   EventSourceName   As String
   EventUserName     As String
   EventComputerName As String
   EventType         As String
   EventDescription  As String
   EventData         As String
   EventCategory     As Integer
   EventRecordNum    As Long
   EventID           As Long
End Type
Global colEventRecord() As EventRecord


' -------------------------------------------------
' Constants for event log types
' -------------------------------------------------
Public Const EVNT_SYSTEM = "System"
Public Const EVNT_APP = "Application"
Public Const EVNT_SECURITY = "Security"


' -------------------------------------------------
' Constants for different types of events
' -------------------------------------------------
Public Const Event_Type_Info = "Information"
Public Const Event_Type_Warning = "Warning"
Public Const Event_Type_Error = "Error"
Public Const Event_Type_Success_Audit = "Audit Success"
Public Const Event_Type_Failure_Audit = "Audit Failure"

Public Const EVENTLOG_ERROR_TYPE = &H1
Public Const EVENTLOG_WARNING_TYPE = &H2
Public Const EVENTLOG_INFORMATION_TYPE = &H4
Public Const EVENTLOG_AUDIT_SUCCESS = &H8
Public Const EVENTLOG_AUDIT_FAILURE = &H10


' -------------------------------------------------
' Constants for different types of event filters
' -------------------------------------------------
Public Const Filter_Type_None = 0
Public Const Filter_Type_TimeBefore = 1
Public Const Filter_Type_TimeAfter = 2
Public Const Filter_Type_EventType = 3
Public Const Filter_Type_Source = 4
Public Const Filter_Type_Category = 5
Public Const Filter_Type_Computer = 6
Public Const Filter_Type_EventID = 7


' -------------------------------------------------
' Error numbers possibly returned from EventLog
'   Server
' -------------------------------------------------
Public Const ERR_LOGTYPE_NOT_SET = 1011 + vbObjectError
Public Const ERR_SOURCENAME_NOT_SET = 1012 + vbObjectError
Public Const ERR_BAD_INDEX = 1013 + vbObjectError
Public Const ERR_FAILED_OPEN_REGISTRY_KEY = 1014 + vbObjectError
Public Const ERR_FAILED_READ_REGISTRY_KEY = 1015 + vbObjectError
Public Const ERR_RESOURCE_DATA_NOT_FOUND = 1016 + vbObjectError
Public Const ERR_READING_EVENT_LOG = 1017 + vbObjectError
Public Const ERR_LOG_NOT_OPENED = 1018 + vbObjectError
Public Const ERR_FAILED_SET_LOG_TYPE = 1019 + vbObjectError

Sub Main()
   frmEventLogs.Show
End Sub

' -------------------------------------------------
' Call the OLE server to read the selected Event
'   Log, in the selected direction, with the
'   selected filter
' -------------------------------------------------
Public Sub ReadEventLog()
   Dim plngRtn       As Long
   Dim plngLoop      As Long
   Dim plngEventCnt  As Long
   Dim pItem         As ListItem
   Dim pEventLog     As EventLogs
   
   
   ' ----------------------------------------------
   ' Insubstantiate the object
   ' ----------------------------------------------
   Set pEventLog = New EventLogs
   
   
   ' ----------------------------------------------
   ' Set filter if it has been set
   ' ----------------------------------------------
   If mvntFilterType <> CVar(Filter_Type_None) Then
      pEventLog.EventFilter(CLng(mvntFilterText)) = mvntFilterType
   End If
   
      
   ' ----------------------------------------------
   ' Set the forward/backwards read
   ' ----------------------------------------------
   If frmEventLogs.optDirectionToRead(0).Value = True Then
      pEventLog.EventReadLogForward = False
   Else
      pEventLog.EventReadLogForward = True
   End If
   
   
   ' ----------------------------------------------
   ' Set data over 256 bytes *NOT* to be converted
   '   to Hex
   ' ----------------------------------------------
   pEventLog.EventDataReturnHex = False
   
   
   ' ----------------------------------------------
   ' Get the settings from the form
   ' ----------------------------------------------
   pEventLog.EventTypeLog = frmEventLogs.LogType
   If pEventLog.OpenAnyEventLog(frmEventLogs.ServerName) = False Then
      
      MsgBox "Error: " & pEventLog.LastEventErrorDescription, vbExclamation, "Error Opening Log"
      frmEventLogs.lblNumEventRecs = "Number Event Records: 0"
      Set pEventLog = Nothing
      Exit Sub
   End If

      
   ' ----------------------------------------------
   ' Read the event log
   ' ----------------------------------------------
   If pEventLog.ReadEventEntries = False Then
      MsgBox "Error: " & pEventLog.LastEventErrorDescription, vbExclamation, "Error Reading Log"
      frmEventLogs.lblNumEventRecs = "Number Event Records: 0"
      Set pEventLog = Nothing
      Exit Sub
   End If

   
   ' ----------------------------------------------
   ' Get a count of the number of records, and
   '   display it on the main form
   ' ----------------------------------------------
   plngEventCnt = pEventLog.CountEventRecords
   frmEventLogs.lblNumEventRecs = "Number Event Records: " & CStr(plngEventCnt)
   If plngEventCnt = 0 Then
      Set pEventLog = Nothing
      Exit Sub
   End If
   
   
   ' ----------------------------------------------
   ' Redim the global type array to receive the
   '   data from the OLE properties.
   ' ----------------------------------------------
   If plngEventCnt > 0 Then
      ReDim colEventRecord(1 To plngEventCnt) As EventRecord
   End If
   
   
   ' ----------------------------------------------
   ' Cycle thru the properties in the collection
   '   in the OLE server
   ' ----------------------------------------------
   If plngEventCnt > 0 Then
      For plngLoop = 1 To plngEventCnt
         
         Screen.MousePointer = vbHourglass
         
         ' ----------------------------------------
         ' Fill in the type array
         ' ----------------------------------------
         colEventRecord(plngLoop).EventTimeWritten = pEventLog.EventTimeWritten(plngLoop)
         colEventRecord(plngLoop).EventSourceName = pEventLog.EventSourceName(plngLoop)
         colEventRecord(plngLoop).EventUserName = pEventLog.EventUserName(plngLoop)
         colEventRecord(plngLoop).EventComputerName = pEventLog.EventComputerName(plngLoop)
         colEventRecord(plngLoop).EventType = pEventLog.EventType(plngLoop)
         colEventRecord(plngLoop).EventDescription = pEventLog.EventDescription(plngLoop)
         colEventRecord(plngLoop).EventData = pEventLog.EventData(plngLoop)
         colEventRecord(plngLoop).EventCategory = pEventLog.EventCategory(plngLoop)
         colEventRecord(plngLoop).EventRecordNum = pEventLog.EventRecordNum(plngLoop)
         colEventRecord(plngLoop).EventID = pEventLog.EventID(plngLoop)
        
         Set pItem = frmEventLogs.lstviewEventRecords.ListItems.Add()
         Select Case colEventRecord(plngLoop).EventType
            Case Event_Type_Info
               pItem.SmallIcon = 1
            Case Event_Type_Error
               pItem.SmallIcon = 2
            Case Event_Type_Warning
               pItem.SmallIcon = 3
            Case Event_Type_Failure_Audit
               pItem.SmallIcon = 4
            Case Event_Type_Success_Audit
               pItem.SmallIcon = 5
         End Select
         
         pItem.Text = Space$(2) & colEventRecord(plngLoop).EventRecordNum
         pItem.SubItems(1) = colEventRecord(plngLoop).EventID
         pItem.SubItems(2) = colEventRecord(plngLoop).EventTimeWritten
         
      Next plngLoop
   End If
   
   
   ' ----------------------------------------------
   ' Release the server
   ' ----------------------------------------------
   Set pEventLog = Nothing

End Sub


' -------------------------------------------------
' Allow an outside form to *set* the values, and
'   the ReadEventLog proc can read these 2
'   module-level variables
' -------------------------------------------------
Public Property Let FilterItem(xvntFilterType As Variant, xvntFilterText As Variant)
   mvntFilterType = xvntFilterType
   mvntFilterText = xvntFilterText
End Property
