Attribute VB_Name = "WAVEMIX"
Option Explicit
'------------------------------------------------------------
' WAVEMIX.BAS
' This module contains declarations for all the functions
' in the WaveMix DLL, and provides some higher-level Basic
' functions to make using WaveMix simpler.
'------------------------------------------------------------
Global hMixSession As Long
Global lpWaveMix() As Long
Global WaveHandle As Long
Global WAVMIX_Quiet As Integer

Global Const WAVEMIX_MAXCHANNELS = 8

Type tChannelInfo
    Loops As Long
    WaveFile As String
End Type

Type tWaveMixInfo
   wSize As Integer
   bVersionMajor As String * 1
   bVersionMinor As String * 1
   szDate(12) As String
   dwFormats As Long
End Type

Type tMixConfig
    wSize As Integer
    dwFlagsLo As Integer
    dwFlagsHi As Integer
    wChannels As Integer
    wSamplingRate As Integer
End Type

Private Type tMixPlayParams
    wSize         As Integer
    hMixSessionLo As Integer
    hMixSessionHi As Integer
    iChannelLo    As Integer
    iChannelHi    As Integer
    lpMixWaveLo   As Integer
    lpMixWaveHi   As Integer
    hWndNotifyLo  As Integer
    hWndNotifyHi  As Integer
    dwFlagsLo     As Integer
    dwFlagsHi     As Integer
    wLoops        As Integer
End Type

Declare Function WaveMixInit Lib "WAVMIX32.DLL" () As Long
Declare Function WaveMixConfigureInit Lib "WAVMIX32.DLL" _
        (lpConfig As tMixConfig) As Long
Declare Function WaveMixActivate Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal fActivate As Integer) As Long
Declare Function WaveMixOpenWave Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal szWaveFilename As String, _
        ByVal hInst As Long, ByVal dwFlags As Long) As Long
Declare Function WaveMixOpenChannel Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal iChannel As Long, _
        ByVal dwFlags As Long) As Long
Declare Function WaveMixPlay Lib "WAVMIX32.DLL" (lpMixPlayParams As Any) As Integer
Declare Function WaveMixFlushChannel Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal iChannel As Integer, _
        ByVal dwFlags As Long) As Integer
Declare Function WaveMixCloseChannel Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal iChannel As Integer, _
        ByVal dwFlags As Long) As Integer
Declare Function WaveMixFreeWave Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long, ByVal lpMixWave As Long) As Integer
Declare Function WaveMixCloseSession Lib "WAVMIX32.DLL" _
        (ByVal hMixSession As Long) As Integer
Declare Sub WaveMixPump Lib "WAVMIX32.DLL" ()
Declare Function WaveMixGetInfo Lib "WAVMIX32.DLL" _
        (lpWaveMixInfo As tWaveMixInfo) As Integer

'------------------------------------------------------------
' These two functions are needed to reverse the word ordering
' of fields in the WaveMixPlay function.
'------------------------------------------------------------
Private Function HiWord(ByVal l As Long) As Integer
    l = l \ &H10000
    HiWord = Val("&H" & Hex$(l))
End Function

Private Function LoWord(ByVal l As Long) As Integer
    l = l And &HFFFF&
    LoWord = Val("&H" & Hex$(l))
End Function

Sub WAVMIX_SetFile(FileName As String, AChannel As Long)
'------------------------------------------------------------
' Assign a new wave file, FileName, to the specified channel,
' AChannel.  If this channel is currently assigned another
' wave file, stop playing the channel and free the active
' wave file.
'------------------------------------------------------------
Dim rc As Long
    
    If WAVMIX_Quiet Then Exit Sub
    
    If AChannel > UBound(lpWaveMix) Then
        ReDim Preserve lpWaveMix(AChannel)
        WaveHandle = AChannel
    End If
    
    ' If another wave is currently assigned to this
    ' channel, free it.
    If lpWaveMix(AChannel) <> 0 Then
        WAVMIX_StopChannel AChannel
        rc = WaveMixFreeWave(hMixSession, lpWaveMix(AChannel))
    End If
    
    ' Open the new wave and assign it to this channel.
    lpWaveMix(AChannel) = WaveMixOpenWave(hMixSession, FileName, 0, 0)
    rc = WaveMixOpenChannel(hMixSession, AChannel, 0)
End Sub


Sub WAVMIX_Close()
'------------------------------------------------------------
' Stop playing all channels and free all wave files, then
' close down this WaveMix session.
'------------------------------------------------------------
Dim rc As Long
Dim i As Integer
    
    If WAVMIX_Quiet Then Exit Sub

    If (hMixSession <> 0) Then
        For i = 0 To UBound(lpWaveMix)
            If lpWaveMix(i) <> 0 Then
                WAVMIX_StopChannel CLng(i)
                rc = WaveMixFreeWave(hMixSession, lpWaveMix(i))
            End If
        Next
        rc = WaveMixCloseSession(hMixSession)
        hMixSession = 0
    End If
End Sub

Function WAVMIX_InitMixer() As Integer
'------------------------------------------------------------
' Initialize and activate the WaveMix DLL.
'------------------------------------------------------------
Dim rc As Long
Dim config As tMixConfig

    If WAVMIX_Quiet Then Exit Function

    WaveHandle = 0
    ReDim lpWaveMix(0)
    ChDir App.Path
    
    config.wSize = Len(config)
    config.dwFlagsHi = 1
    config.dwFlagsLo = 0
    'Allow stereo sound
    config.wChannels = 2
    hMixSession = WaveMixConfigureInit(config)
    rc = WaveMixActivate(hMixSession, True)

    If (rc <> 0) Then
        WAVMIX_InitMixer = False
        Call WaveMixCloseSession(hMixSession)
        hMixSession = 0
    Else
        WAVMIX_InitMixer = True
    End If
End Function

Sub WAVMIX_StopChannel(ByVal ChannelNum As Long)
'------------------------------------------------------------
' Stop playing the specified channel.
'------------------------------------------------------------
Dim rc As Integer

    If WAVMIX_Quiet Then Exit Sub
    If (hMixSession = 0) Then Exit Sub
    
    rc = WaveMixFlushChannel(hMixSession, ChannelNum, 0)
End Sub

Sub WAVMIX_Activate(Activate As Long)
'------------------------------------------------------------
' Activate the WaveMix DLL.
'------------------------------------------------------------
Dim rc As Integer

    If WAVMIX_Quiet Then Exit Sub
    If (hMixSession = 0) Then Exit Sub

    rc = WaveMixActivate(hMixSession, Activate)
End Sub

Sub WAVMIX_PlayChannel(ChannelNum As Long, LoopWave As Long)
'------------------------------------------------------------
' Play a specified channel, and indicate whether the sound
' should be looped.
'------------------------------------------------------------
Dim params As tMixPlayParams
Dim rc As Long

    If WAVMIX_Quiet Then Exit Sub
    If ChannelNum > UBound(lpWaveMix) Then Exit Sub
    If (hMixSession = 0) Then Exit Sub

    params.wSize = Len(params)
    params.hMixSessionLo = LoWord(hMixSession)
    params.hMixSessionHi = HiWord(hMixSession)
    params.iChannelLo = LoWord(ChannelNum)
    params.iChannelHi = HiWord(ChannelNum)
    params.lpMixWaveLo = LoWord(lpWaveMix(ChannelNum))
    params.lpMixWaveHi = HiWord(lpWaveMix(ChannelNum))
    params.hWndNotifyLo = 0
    params.hWndNotifyHi = 0
    params.dwFlagsHi = 5
    params.dwFlagsLo = 0
    params.wLoops = LoopWave
    rc = WaveMixPlay(params)
End Sub

